import time
import json

from host import *
from behave import *
from steps import login, interface, common


@when('"{device}" enable WAN-Ping-Response')
@when('enable WAN-Ping-Response on "{device}"')
@given('enable WAN-Ping-Response on "{device}"')
def step_impl(context, device):
    time.sleep(1)
    context.dut[device].go_config().set_ip_ping_response()


@When('disable {interface} interface from "{device}"')
def step_impl(context, interface, device):
    if interface == "WAN":
        context.dut[device].go_config_if_wan().disable_wan_interface()
    elif interface == "BRG":
        context.dut[device].go_config_bridge().disable_brg_interface()
    else:
        raise ValueError(f'inpurt invalid interface {interface}')


@given('modify {interface} ip address on "{device}"')
@when('modify {interface} ip address on "{device}"')
def step_impl(context, interface, device):
    context.dut_info[device]["testbed"][
        f"{interface.lower()}_ip"] = context.table[0][interface.lower()]
    if interface == "WAN":
        context.dut[device].go_config_if_wan().set_wan_interface( \
            ip=context.table[0][interface.lower()], \
            mask=context.table[0]["mask"])
    elif interface == "LAN":
        context.dut[device].go_config_if_lan().set_lan_interface( \
            ip=context.table[0][interface.lower()], \
            mask=context.table[0]["mask"])
    elif interface == "BRG":
        context.dut[device].go_config_bridge().set_brg_interface( \
            ip=context.table[0][interface.lower()], \
            mask=context.table[0]["mask"])
    else:
        raise ValueError(f'inpurt invalid interface {interface}')


@when('"{host}" create {number} web session to "{device}" {intf}')
def step_impl(context, host, device, number, intf):
    if hasattr(context, 'dut_jwt'):
        # Session Clean Up
        if device in context.dut_jwt:
            for jwt in context.dut_jwt[device]:
                context.hosts[host].dut_ui.web_post(
                    uri="api/v1/auth/logout",
                    jwt=jwt,
                    ip=context.dut_info[device]['session']['host'])
        context.dut_jwt[device] = []
    else:
        context.dut_jwt = {}

    web_jwt = []
    for i in range(1, int(number) + 1):
        web_jwt_one = context.hosts[host].dut_ui.web_get_jwt( \
            username=context.dut_info[device]['credential']['username'], \
            password=context.dut_info[device]['credential']['password'], \
            ip=context.dut_info[device]['testbed'][f'{intf.lower()}_ip'])
        if web_jwt_one is not None:
            web_jwt.append(web_jwt_one)
    assert len(web_jwt) == int(
        number), f'Only {len(web_jwt)} is created but {number} is required.'
    context.dut_jwt[device] = web_jwt


@then('"{host}" retrieve "{device}" {intf} uptime with each web session')
def step_impl(context, host, device, intf):
    for jwt in context.dut_jwt[device]:
        ret = context.hosts[host].dut_ui.web_get(
            uri="api/v1/status/systemInformation",
            jwt=jwt,
            ip=context.dut_info[device]['testbed'][f'{intf.lower()}_ip'])
        assert ret != None, f'Fail to retrieve {device} uptime with jwt {jwt}'
        print(f"-----> uptime: {json.loads(ret[0])['uptime']}")


@when('set maximum web session to {number} on "{device}"')
def step_impl(context, number, device):
    context.dut[device].go_config().set_ip_http_max_login_users(number)


@then('"{host}" fail to create {session_seq} web session to "{device}" {intf}')
def step_impl(context, host, device, session_seq, intf):
    session_index = int(session_seq[:-2])
    assert session_index - 1 == len(
        context.dut_jwt[device]
    ), f'assume there are {session_index-1} web sessions but {len(context.dut_jwt[device])}'

    web_jwt_one = context.hosts[host].dut_ui.web_get_jwt( \
        username=context.dut_info[device]['credential']['username'], \
        password=context.dut_info[device]['credential']['password'], \
        ip=context.dut_info[device]['testbed'][f'{intf.lower()}_ip'])
    assert web_jwt_one == None, f'{session_seq} web session should fail but success'



import sys
import time

from host import *
from behave import *
from steps.common import *
from steps.interface import *
from steps.login import *


@given('"{device}" enables login lockout toggle')
def step_impl(context, device):
    cfg = context.dut[device].go_config()
    cfg.LoginLockout(cfg.get_session()).lockout_toggle(True)


@given('"{device}" sets "Login Failure Retry Threshold" to "{num}" times')
def step_impl(context, device, num):
    cfg = context.dut[device].go_config()
    cfg.LoginLockout(cfg.get_session()).lockout_retryThreshold(num)


@given('"{device}" sets "Lockout Duration" to "{num}" minute')
def step_impl(context, device, num):
    cfg = context.dut[device].go_config()
    cfg.LoginLockout(cfg.get_session()).lockout_lockoutTime(num)


@given('"{device}" logins with incorrect password for "{num}" times')
@when('"{device}" logins with incorrect password for "{num}" times')
def step_impl(context, device, num):
    # This is a trick, to avoid iptables PortScan chain dropping our packets.
    # PortScan will drop our packet if 3 attempts within 30 seconds request for TCP connection.
    context.dut[device].main().clear_ssh_telnet_limit()
    if (int(num) > 3):
        raise Exception(
            "Design limitation. Unable to initiate connection attempts more than 3 times within 30 seconds."
        )
    # print('logoutFromDUT 1') #debug
    logoutFromDUT(context, device)
    # print('logoutFromDUT 2') #debug
    for _ in range(int(num)):
        user = 'admin'
        password = 'wrong_pwd'
        context.execute_steps(
            f'''then unable to login "{device}" as {user} with password {password}'''
        )
    context.dut[device].timer = time.time()


@when(
    '"{device}" logins with correct password for one more time after "{num}" seconds'
)
def step_impl(context, device, num):
    s_time = context.dut[device].timer
    print(f"start time = {s_time}")
    print(f"current time = {time.time()}")
    r = int(num) - (time.time() - s_time) + 1  # +1 for get ceil
    print(f"residual time = {r} sec")
    if r > 0:
        time.sleep(r)
    try:
        context.dut[device].main().exit()
    except:
        pass
    try:
        context.dut[device].close()
    except:
        pass
    try:
        context.dut_info[device]['credential']['username'], context.dut_info[device]['credential']['password'] = 'admin', 'moxa'
        loginToDUT(context, device)  # may succeed or fail
    except:
        pass
    finally:
        # restore s_time back to router object
        context.dut[device].timer = s_time


@then('"{device}" is {capable} to login with correct password')
def step_impl(context, device, capable):
    if capable == "unable":
        try:
            context.dut[device].main().show_system(
            )  # suppose to be caught by exception
        except:
            print("OK! Suppose to except!")
        else:
            raise Exception(
                f"The device {device} is supposed to be unable to login")
    elif capable == "able":
        try:
            context.dut[device].main().show_system()  # suppose to work
        except:
            raise Exception(
                f"The device {device} is supposed to be able to login")
    else:
        raise Exception(f"Unsupported flag: {capable}")


@then('"{device}" can see system log "{log_key_word}" for "{num}" times')
def step_impl(context, device, log_key_word, num):
    logs = context.dut[device].main().show_logging_event_log_system()
    count = 0
    for log in logs:
        if log_key_word in log['event']:
            count += 1
    assert count == int(num), f'number of logs not matched {num} times'



from mdc.router.cli.rp_base.lg_router.tn5000 import config
from mdc.router.cli.rp_base.lg_router.tn5000.cli import PROMPT_CONFIG

class ConfigFirewall(config.Config):
    """
    Class for configuring L37 Policy.

        Args:
            session (object): A valid session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """
    def __init__(self, session, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        index = self._args[0] if len(self._args) != 0 else None
        if index is None:
            raise Exception(f"Input parameter is empty!")
        self._s.command_expect(f'firewall {index}')

    def getin(self):
        self._set_default_prompts()
        self._getin()
        return self

    def set_firewall_policy(self,
                            action=None,
                            mode=None,
                            protocol=None,
                            itf=None,
                            sip=None,
                            dip=None,
                            sport=None,
                            dport=None,
                            smac=None,
                            enable_log=None,
                            disable_log=None):
        """
        :type itf: dict
        :type enable_log: list
        :type disable_log: list
        """
        if action in ["accept", "drop"]:
            self._s.command_expect(f"action {action}")
        if mode in ["ip", "mac"]:
            self._s.command_expect(f"mode {mode}")
        if protocol:
            self._s.command_expect(f"protocol {protocol}")
        if itf:
            self._s.command_expect(f'interface {itf["in"]} {itf["out"]}')
        if sip:
            self._s.command_expect(f"src-ip single {sip}")
        if dip:
            self._s.command_expect(f"dst-ip single {dip}")
        if sport:
            self._s.command_expect(f"src-port {sport}")
        if dport:
            self._s.command_expect(f"dst-port {dport}")
        if smac:
            self._s.command_expect(f"src-mac {smac}")
        log_item = ["flash", "syslog", "trap"]
        if isinstance(enable_log, list):
            for i in enable_log:
                if i in log_item:
                    self._s.command_expect(f"logging {i}")
                else:
                    raise ValueError(f'input invalid item: {i}')
        if isinstance(disable_log, list):
            for i in disable_log:
                if i in log_item:
                    self._s.command_expect(f"no logging {i}")
                else:
                    raise ValueError(f'input invalid item: {i}')

        self._s.command_expect("exit")



from behave import *
from steps import common, interface, login


@when(u'set L2TP server on "{device}"')
def step_impl(context, device):
    local_ip = context.table[0]['Local IP']
    end_ip = context.table[0]['Offered IP: End']
    context.start_ip = context.table[0]['Offered IP: Start']

    context.cfg = context.dut[device].go_config()
    context.l2tp_cfg = context.cfg.L2tp(context.cfg.get_session())
    context.l2tp_cfg.set_l2tp_rule(local_ip, context.start_ip, end_ip)


@when(u'set L2TP user on "{device}"')
def step_impl(context, device):
    username = context.table[0]['Username']
    password = context.table[0]['Password']

    context.l2tp_cfg.set_l2tp_user(username, password)


@then(u'the following rule should be on "{device}"s l2tp table')
def step_impl(context, device):
    server_mode = context.table[0]['Server Mode']
    local_ip = context.table[0]['Local IP']
    username = context.table[0]['User Name']
    offered_ip = context.table[0]['Offered IP Range'].replace('-', '').split()
    flag = False

    ret = context.dut[device].main().show_l2tp_setting()[0]
    if ret['server_mode'] == server_mode and ret['local_ip'] == local_ip and \
        ret['start_ip'] == offered_ip[0] and ret['end_ip'] == offered_ip[1] and \
        ret['username'] == username:

        flag = True

    assert flag, f"l2tp is not setting up correctly ->\n expect: {context.table[0]}\n actual: {ret}"


@when(u'"{host}" tries to establish l2tp connection to "{device}"s l2tp server')
def step_impl(context, host, device):
    context.hosts[host].nmcli.intf_setting(managed=True)

    context.con_name = 'test_l2tp'

    ## Create a new host named "l2tp_client" based on HOST_A.
    context.hosts['l2tp_client'] = context.hosts['HOST_A']
    context.host_info['l2tp_client'] = context.host_info['HOST_A']
    context.host_info['l2tp_client']["testbed"]["cur_host"] = context.start_ip

    context.hosts[host].nmcli.create_connection(
        con_type='vpn',
        con_name=context.con_name,
        vpn_type='l2tp',
        gateway=context.dut_info[device]['testbed']['wan_ip'],
        user='user',
        password='admin')
    context.hosts[host].nmcli.activate_connection(con_name=context.con_name,
                                                  ifname='eth1')


@then(u'verify "{host}" is assigned the IP "{ip}" with interface "{intf}"')
def step_impl(context, host, ip, intf):
    ret = context.hosts[host].shellcmd.show_network(dev=intf)
    ret = context.hosts[host].shellcmd.show_network()

    assert 'does not exist' not in ret, f'{intf} does not exist'
    assert ip in ret, f'Interface "{intf}" has no ip "{ip}"'


@given(u'disable and clean up l2tp connection on "{host}"')
def step_impl(context, host):

    context.hosts[host].nmcli.deactivated_connection(con_name=context.con_name)
    context.hosts[host].nmcli.delete_connection(con_name=context.con_name)
    context.hosts[host].nmcli.intf_setting(managed=False)



import time

from host import *
from behave import *


@given('add a static route with unknown destination ip for "{host}"')
def step_impl(context, host):
    context.hosts[host].shellcmd.add_route(
        dip=f'{context.table[0]["st_route_dip"]}',
        gw=f'{context.table[0]["st_route_next_hop"]}',
        dev=context.host_info[host]["nic"],
        mask='32')


@when('set "{device}" a static route rule with parameters and status {status}')
@given('set "{device}" a static route rule with parameters and status {status}')
def step_impl(context, device, status):
    context.dut[device].go_config().command(
        f'ip route static TCR-664 {context.table[0]["destination_ip"]} {context.table[0]["mask"]} {context.table[0]["next_hop"]} {context.table[0]["metric"]}'
    )
    context.dut[device].go_config().command(
        f'ip route static TCR-664 {status.lower()}')


@when('send test packet with the destination ip form "{host1}" to "{host2}"')
def step_impl(context, host1, host2):
    context.hosts[host2].tshark.start(interface=context.host_info[host2]['nic'])
    context.hosts[host1].shellcmd._ssh.command(
        f'ping {context.table[0]["dip"]} -c 4')
    time.sleep(10)


@then('check if "{host1}" {action} receive test packet from "{host2}"')
def step_impl(context, action, host1, host2):
    context.hosts[host1].tshark.stop()
    rcv_pkt = context.hosts[host1].tshark.retrieve()
    hit = 0
    hit = rcv_pkt.count(
        f"{context.host_info[host2]['testbed']['cur_host']} → 192.168.129.30 ICMP"
    )
    print(f"[RSV PKT {hit}:]" + rcv_pkt)
    if "can" == action:
        assert hit != 0, f'test packets are not received! rcv_pkt: {rcv_pkt}'
    elif "can not" == action:
        assert hit == 0, f'{hit} test packets are received! rcv_pkt: {rcv_pkt}'



from mdc.router.cli.rp_base.lg_router.tn5000 import config
from mdc.router.cli.rp_base.lg_router.tn5000.cli import PROMPT_CONFIG

class ConfigBrg(config.Config):
    """
    Class for configuring Bridge interfaces.

        Args:
            session (object): A valid session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        self._s.command_expect(f'interface bridge')

    def getin(self):
        self._set_default_prompts()
        self._getin()
        return self

    def set_brg_interface(self, name=None, ip=None, mask=None):
        if name:
            self._s.command_expect(f'name {name}')
        if ip and mask:
            self._s.command_expect(f'ip address {ip} {mask}')
        return self

    def disable_brg_interface(self):
        self._s.command_expect(f'shutdown')
        return self



import time
import ipaddress

from behave import *
from utils.input_chk import chk_is_private_ip
from mdc.router.cli.rp_base.ng_router.base import Base
from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000

'''
    Description:
        To specify DUT interface configuration, add table to the step.
        If table or table header is missing, network configuration is loaded from topology JSON.

    Example Tables:
        | wan             | mask          | wan_vlan | wan_port | wan_gw        |
        | 192.168.128.254 | 255.255.255.0 | 2        | 8        | 192.168.128.1 |

        | brg             | mask          | brg_vlan | brg_port |
        | 192.168.126.254 | 255.255.255.0 | 4040     | 1,8      |
'''
def get_intf_ip(context, interface, device):
    # Interface IP Address
    if context.table and interface.lower() in context.table.headings:
        dut_iface_ip = context.table[0][interface.lower()]
    else:
        dut_iface_ip = context.dut_info[device]["testbed"][
            f"{interface.lower()}_ip"]
    return dut_iface_ip
def get_intf_port(context, interface, device):
    # Interface Port
    if context.table and f"{interface.lower()}_port" in context.table.headings:
        if interface == 'BRG':
            dut_iface_port = context.table[0][
                f"{interface.lower()}_port"].split(',')
        else:
            dut_iface_port = context.table[0][f"{interface.lower()}_port"]
    else:
        dut_iface_port = context.dut_info[device]["testbed"][
            f"{interface.lower()}_port"]
    return dut_iface_port
def get_intf_vlan(context, interface, device):
    # Interface Vlan
    if context.table and f"{interface.lower()}_vlan" in context.table.headings:
        dut_iface_vlan = context.table[0][f"{interface.lower()}_vlan"]
    else:
        dut_iface_vlan = context.dut_info[device]["testbed"][
            f"{interface.lower()}_vlan"]
    return dut_iface_vlan
def get_intf_mask(context, interface, device):
    # Interface Netmask
    if context.table and "mask" in context.table.headings:
        dut_iface_mask = context.table[0]["mask"]
    else:
        dut_iface_mask = context.dut_info[device]["testbed"]["mask"]
    return dut_iface_mask
def get_intf_gw(context, interface, device):
    # Interface Default GW
    if context.table and f"{interface.lower()}_gw" in context.table.headings:
        dut_iface_gw = context.table[0][f"{interface.lower()}_gw"]
    else:
        dut_iface_gw = None
    return dut_iface_gw
def set_interface(context, interface, device, dut_iface_port, 
                  dut_iface_ip, dut_iface_mask, dut_iface_gw, dut_iface_vlan):
    if interface == 'WAN':
        context.dut[device].go_config().set_vlan_id(dut_iface_vlan)
        context.dut[device].go_config_if_ethernet(
            f'1/{dut_iface_port}').set_switchport(dut_iface_vlan)
        context.dut[device].go_config_if_wan().set_wan_interface(
            ip=dut_iface_ip, mask=dut_iface_mask, bind_vlan=dut_iface_vlan, gateway=dut_iface_gw)
    elif interface == 'LAN':
        if chk_is_private_ip(dut_iface_ip) is False:
            raise ValueError(
                f"The interface IP {dut_iface_ip} on {interface} is not a private IP!"
            )
        context.dut[device].go_config_if_ethernet(
            f'1/{dut_iface_port}').set_switchport(dut_iface_vlan)
        context.dut[device].go_config_if_lan().set_lan_interface(
            ip=dut_iface_ip, mask=dut_iface_mask, bind_vlan=dut_iface_vlan)
        
    elif interface == 'BRG':
        if chk_is_private_ip(dut_iface_ip) is False:
            raise ValueError(
                f"The interface IP {dut_iface_ip} on {interface} is not a private IP!"
            )
        if issubclass(context.dut[device]._model, tn5000):
            context.dut[device].go_config_bridge().set_brg_interface(
                name="BRG", ip=dut_iface_ip, mask=dut_iface_mask)
            for p in dut_iface_port:
                dut_iface_vlan = int(dut_iface_vlan) + int(p)
                context.dut[device].go_config_if_ethernet(f'1/{p}'). \
                                    set_switchport(dut_iface_vlan) .\
                                    set_bridge_group(group=1)
        else:
            context.dut[device].go_config_bridge().set_brg_interface(
                ip=dut_iface_ip, mask=dut_iface_mask)
            for p in dut_iface_port:
                dut_iface_vlan = int(dut_iface_vlan) + int(p)
                context.dut[device].go_config_if_ethernet(f'1/{p}'). \
                                    set_switchport(dut_iface_vlan) .\
                                    set_bridge_group()
    else:
        raise ValueError(f'input interface:{interface} is not exist')
def check_interface(context, interface, device, dut_iface_ip):
    interface_cmd = {
        "wan": "wan",
        "lan": "lan",
        "brg": "bridge"
    }[interface.lower()]
    count = 0
    ret = context.dut[device].main().show_interface(interface_cmd)
    while ret[0]['ip_address'].strip() not in dut_iface_ip.strip(
    ) and count < 60:
        print(ret[0]['ip_address'], dut_iface_ip)

        time.sleep(1)
        count += 1
        ret = context.dut[device].main().show_interface(interface_cmd)

@given(u'set {interface} interface on "{device}" to access internet')
@when(u'set {interface} interface on "{device}" to access internet')
def step_impl(context, interface, device):
    dut_iface_ip = get_intf_ip(context, interface, device)
    dut_iface_vlan = get_intf_vlan(context, interface, device)
    dut_iface_port = context.dut_info[device]["testbed"]["internet_port"]
    dut_iface_mask = get_intf_mask(context, interface, device)
    dut_iface_gw = get_intf_gw(context, interface, device)
    set_interface(context, interface, device, dut_iface_port, 
                  dut_iface_ip, dut_iface_mask, dut_iface_gw, dut_iface_vlan)
    check_interface(context, interface, device, dut_iface_ip)
    
@given(u'set {interface} interface on "{device}" with dhcp mode')
@When(u'set {interface} interface on "{device}" with dhcp mode')
def step_impl(context, interface, device):
    dut_iface_vlan = get_intf_vlan(context, interface, device)
    dut_iface_port = context.dut_info[device]["testbed"]["internet_port"]

    context.dut[device].go_config().set_vlan_id(dut_iface_vlan)
    context.dut[device].go_config_if_ethernet(
            f'1/{dut_iface_port}').set_switchport(dut_iface_vlan)
    context.dut[device].go_config_if_wan().ip_address_dhcp(bind_vlan=dut_iface_vlan)
    for i in range(30):
        time.sleep(1)
        ret = context.dut[device].main().show_interface('wan')
        if '0.0.0.0' not in ret[0]['ip_address']:
            break

@given(u'set {interface} interface on "{device}"')
@When(u'set {interface} interface on "{device}"')
def step_impl(context, interface, device, internet=None):
    dut_iface_ip = get_intf_ip(context, interface, device)
    dut_iface_vlan = get_intf_vlan(context, interface, device)
    dut_iface_port = get_intf_port(context, interface, device)
    dut_iface_mask = get_intf_mask(context, interface, device)
    dut_iface_gw = get_intf_gw(context, interface, device)
    set_interface(context, interface, device, dut_iface_port, 
                  dut_iface_ip, dut_iface_mask, dut_iface_gw, dut_iface_vlan)
    check_interface(context, interface, device, dut_iface_ip)
    
@given(u'set "{host}" ip address within "{device}"s {interface} subnet')
@when(u'set "{host}" ip address within "{device}"s {interface} subnet')
def step_impl(context, interface, host, device):
    # IP Address
    if context.table and interface.lower() in context.table.headings:
        host_testbed_ip = context.table[0][interface.lower()]
    else:
        host_testbed_ip = context.host_info[host]["testbed"][
            f"{interface.lower()}_host"]

    # Subnet Mask
    if context.table and "mask" in context.table.headings:
        host_testbed_mask = context.table[0]["mask"]
    else:
        host_testbed_mask = context.host_info[host]["testbed"]["mask"]

    # Network name (subnet)
    host_testbed_subnet = str(
        ipaddress.ip_network(f'{host_testbed_ip}/{host_testbed_mask}',
                             strict=False)).split('/')[0]

    # Gateway IP
    if context.table and "gateway" in context.table.headings:
        host_testbed_gw = context.table[0]["gateway"]
    else:
        host_testbed_gw = context.host_info[host]["testbed"][
            f"{interface.lower()}_gw"]

    context.host_info[host]["testbed"]["cur_host"] = host_testbed_ip
    context.host_info[host]["testbed"]["cur_subnet"] = host_testbed_subnet
    context.host_info[host]["testbed"]["cur_gw"] = host_testbed_gw

    context.hosts[host].shellcmd.set_network(ip=host_testbed_ip,
                                        mask=host_testbed_mask,
                                        dev=context.host_info[host]['nic'])
    print(context.hosts[host].shellcmd.show_network(dev=context.host_info[host]['nic']))

'''
    Description:
        Since multiple IP addresses are not defined in topology JSON, step table is required.

    Example Table:
        | lan            | mask          |
        | 192.168.127.10 | 255.255.255.0 |
        | 192.168.127.20 | 255.255.255.0 |
        | 192.168.127.30 | 255.255.255.0 |
'''


@given(u'set "{host}" multiple ip address within "{device}"s {interface} subnet'
      )
def step_impl(context, interface, host, device):
    assert context.table is not None

    context.host_info[host]['testbed']['cur_host'] = context.table[0][
        interface.lower()]
    context.host_info[host]["testbed"]["cur_subnet"] = str(
        ipaddress.ip_network(
            f'{context.host_info[host]["testbed"]["cur_host"]}/{context.host_info[host]["testbed"]["mask"]}',
            strict=False)).split('/')[0]
    context.host_info[host]["testbed"]["cur_gw"] = context.host_info[host][
        "testbed"][f"{interface.lower()}_gw"]

    ip_list = []
    mask_list = []
    for if_list in context.table:
        ip_list.append(if_list[interface.lower()])
        mask_list.append(if_list['mask'])

    context.hosts[host].shellcmd.set_networks(ip=ip_list,
                                         mask=mask_list,
                                         dev=context.host_info[host]['nic'])
    print(context.hosts[host].shellcmd.show_network())


@given(u'add static route for "{host1}" routing to "{host2}"')
@when(u'add static route for "{host1}" routing to "{host2}"')
def step_impl(context, host1, host2):
    if context.host_info[host2]["testbed"]["cur_subnet"] != context.host_info[host1]["testbed"]["cur_subnet"]:
        context.hosts[host1].shellcmd.add_route(
            dip=context.host_info[host2]["testbed"]["cur_subnet"],
            mask=context.host_info[host2]["testbed"]["mask"],
            gw=context.host_info[host1]["testbed"]["cur_gw"],
            dev=context.host_info[host1]['nic'])
        print(context.hosts[host1].shellcmd.show_route())


@when(u'add static route for "{host}" with the following rule')
def step_impl(context, host):
    context.hosts[host].shellcmd.add_route(
        dip=context.table[0]["Subnet"],
        mask=context.table[0]["Mask"],
        gw=context.table[0]["Gateway"],
        dev=context.host_info[host]['nic'])
    print(context.hosts[host].shellcmd.show_route())


@When('set network "{port}" on "{itf}" interface binding to VLAN '
      '"{vlan_id}" on "{device}"')
@Given('set network "{port}" on "{itf}" interface binding to VLAN '
       '"{vlan_id}" on "{device}"')
def step_impl(context, port, itf, vlan_id, device):
    context.dut[device].go_config().set_vlan_id(vlan_id)
    if itf == 'WAN':
        if ',' in context.dut_info[device]["testbed"][port]:
            wan_port_list = context.dut_info[device]["testbed"][port].split(',')
        else:
            wan_port_list = [context.dut_info[device]["testbed"][port]]

        for wan_port in wan_port_list:
            context.dut[device].go_config_if_wan().set_wan_interface(
                context.dut_info[device]["testbed"]["wan_ip"],
                context.dut_info[device]["testbed"]["wan_netmask"],
                bind_vlan=vlan_id)
            context.dut[device].go_config_if_ethernet(
                f'1/{wan_port}').set_switchport(vlan_id)

        r = context.dut[device].main().show_interface(itf)
        count = 0

        while context.dut_info[device]["testbed"]["wan_ip"] not in r[0][
                'ip_address'] and count < 60:
            time.sleep(1)

            r = context.dut[device].main().show_interface(itf)

            count += 1

    elif itf == 'LAN':
        context.dut[device].go_config_if_ethernet(
            context.dut_info[device]["testbed"][port]).set_switchport(vlan_id)
        context.dut[device].go_config_if_vlan(vlan_id).set_ip(
            context.dut_info[device]["testbed"]["lan_ip"],
            context.dut_info[device]["testbed"]["lan_netmask"])
        context.dut[device].go_config_if_vlan(vlan_id).set_name(
            f'VLAN-{vlan_id}')

        r = context.dut[device].main().show_interface('vlan', vid=vlan_id)
        count = 0

        while context.dut_info[device]["testbed"]["lan_ip"] not in r[0][
                'ip_address'] and count < 60:
            time.sleep(1)
            r = context.dut[device].main().show_interface('vlan', vid=vlan_id)

            count += 1



from behave import *

from steps import common, interface, login


@when(
    'use "username" with "incorrect password" to login for the "ordinal" time on "{device}"'
)
def step_impl(context, device):
    login.logoutFromDUT(context, device)
    for row in context.table:
        user = row['username']
        password = row['incorrect password']
        context.execute_steps(
            f'''then unable to login "{device}" as {user} with password {password}'''
        )
    context.dut_info[device]['credential']['username'], context.dut_info[device]['credential']['password'] = 'admin', 'moxa'
    context.dut[device].close()
    login.loginToDUT(context, device) 



import time
import re
from session import SshSession
from utils.decorator import root_access_required

class MailServerService:
    """
    A class for managing a mail server using Docker Compose.

        Attributes:
            _ssh (SshSession): An SSH session object for connecting to the server.
            _dir (str): The directory where the mail server files are stored.
            _sudo_pwd (str): The password for sudo access.
            _hostname (str): The hostname of the mail server.
            _mutt (MuttService): A MuttService object for managing the Mutt email client.
            DMS_GITHUB_URL (str): The URL for the Docker Mail Server GitHub repository.

        Methods:
            __init__(self, ssh_session: SshSession, sudo_paswd: str): Initializes the MailServerService object.
            _check_conneted(self): Checks if the SSH session is connected.
            check_env(self): Checks the environment for the mail server and creates necessary files and directories if they don't exist.
            start(self): Starts the mail server by running docker-compose up command and waits for the mailserver to be up-to-date.
            clean_up(self, service): Cleans up the specified service.
            _make_compose_file(self): Downloads the compose.yaml file from the DMS_GITHUB_URL and extracts the hostname from it.
            _make_env_file(self): Downloads the mailserver.env file from the DMS GitHub URL and saves it to the current directory.
            add_mailserver_account(self, user, paswd): Adds a new email account to the mail server.
            set_mailserver(self): Sets the mail server by removing $myhostname to prevent hostname conflict.
            set_mutt_service(self, ip, user="mutt", paswd="adminmoxa"): Sets up and starts the Mutt email client service.
    """
    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._dir = "$HOME/mailserver"
        self._sudo_pwd = sudo_paswd
        self._hostname = "mail.example.com"
        self._mutt = MuttService(self._ssh, self._sudo_pwd)
        self.DMS_GITHUB_URL="https://raw.githubusercontent.com/docker-mailserver/docker-mailserver/master"

    def _check_conneted(self):
        """
        Check if the environment is valid.

            Raises:
                ConnectionError: If the SSH session is not connected or if no docker image cincan/tshark is found.
        """
        if not self._ssh.is_connected():
            raise ConnectionError('SSH session not connected')
        
    @root_access_required
    def check_env(self):
        """
        Checks the environment for the mail server and creates necessary files and directories if they don't exist.
        Also installs docker-compose if it's not already installed.
        """
        self._check_conneted()
        self._ssh.command(f'cd $HOME')
        output = self._ssh.command('ls')
        if "mailserver" not in output:
            self._ssh.command(f'mkdir {self._dir}')
        self._ssh.command(f'cd {self._dir}')
        output = self._ssh.command('ls')
        if "compose.yaml" not in output:
            self._make_compose_file()
        if "mailserver.env" not in output:
            self._make_env_file()
        output = self._ssh.command(f'command -v docker-compose')
        if "/docker-compose" not in output:
            self._ssh.command("N")
            self._ssh.command('curl -L "https://github.com/docker/compose/releases/download/1.29.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/local/bin/docker-compose',
                                exact_prompts=['100'],
                                timeout=300)
            self._ssh.command('chmod +x /usr/local/bin/docker-compose')

    @root_access_required
    def start(self):
        """
        Starts the mail server by running docker-compose up command and waits for the mailserver to be up-to-date.
        """
        self._ssh.command(f'cd {self._dir}')
        self._ssh.command(f'docker-compose up -d', 
                            exact_prompts=['done', 'mailserver is up-to-date'], 
                            timeout=300)
        time.sleep(5)

    @root_access_required
    def clean_up(self, service):
        """
        Cleans up the specified service.

            Args:
                service (str): The name of the service to clean up. Can be "mail_server" or "mutt".

            Raises:
                ValueError: If the specified service is not "mail_server" or "mutt".
        """
        if service == "mail_server":
            self._ssh.command(f'cd {self._dir}')
            self._ssh.command(f'docker-compose down', 
                            exact_prompts=['Removing network mailserver_default'], 
                            timeout=300)
            self._ssh.command(f'rm -r {self._dir}')
        elif service == "mutt":
            self._mutt.clean_up()
        else:
            raise ValueError(f"Unknown service: {service}")
        
    def _make_compose_file(self):
        """
        Downloads the compose.yaml file from the DMS_GITHUB_URL and extracts the hostname from it.
        """
        self._ssh.command(f'cd {self._dir}')
        self._ssh.command(f'wget "{self.DMS_GITHUB_URL}/compose.yaml"',
                  exact_prompts=['100%'],
                  timeout=300)
                  
        output = self._ssh.command("cat compose.yaml")
        match = re.search(r"hostname:\s*(\S+)", output)
        self._hostname = match.group(1)

    def _make_env_file(self):
        """
        Downloads the mailserver.env file from the DMS GitHub URL and saves it to the current directory.
        """
        self._ssh.command(f'wget "{self.DMS_GITHUB_URL}/mailserver.env"',
                            exact_prompts=['100%'],
                            timeout=300)
    
    def add_mailserver_account(self, user, paswd):
        """
        Adds a new email account to the mail server.

            Args:
                user (str): The username for the new email account.
                paswd (str): The password for the new email account.
        """
        self._ssh.command(f'''docker exec -it mailserver bash -c "setup email add {user}@{self._hostname} {paswd}"''')
    
    def set_mailserver(self):
        """
        Sets the mail server by removing $myhostname to prevent hostname conflict.
        """
        # remove $myhostname to prevent hostname conflict
        self._ssh.command(f'''docker exec -it mailserver bash -c "postconf -e 'mydestination = localhost.\$mydomain, localhost'"''')

    def set_mutt_service(self, ip, user="mutt", paswd="adminmoxa"):
        """
        Sets up and starts the Mutt email client service.

            Args:
                ip (str): The IP address of the local host.
                user (str): The username for the Mutt email client. Default is "mutt".
                paswd (str): The password for the Mutt email client. Default is "adminmoxa".
        """
        self._mutt.check_env()
        self._mutt.make_muttrc_conf(hostname=self._hostname, user=user, paswd=paswd)
        self._mutt.set_local_hosts(ip=ip, hostname=self._hostname)
        self._mutt.start()
        time.sleep(10)

    @root_access_required
    def set_local_hosts(self, ip):
        """
        Sets the local hosts file on the mail server to include the given IP address and hostname.

            Args:
                ip (str): the IP address to add to the hosts file
        """
        output = self._ssh.command(f"cat /etc/hosts")
        if self._hostname not in output:
            self._ssh.command(f"echo '{ip}  {self._hostname}' >> /etc/hosts")

    @root_access_required
    def get_email(self):
        """
        Retrieves the latest email from the mail server.

            Returns:
                str: The contents of the latest email.
        """
        dir = "/var/mail/mail.example.com/mutt/new"
        output = self._ssh.command(f'docker exec -it mailserver bash -c "ls {dir}"')
        tmp = re.search(r"'(\S+)'", output).group(1)
        output = self._ssh.command(f'docker exec -it mailserver bash -c "cat {dir}/{tmp}"')

        return output
    
class MuttService:
    """
    A class representing a Mutt service.

        Attributes:
            _ssh (SshSession): The SSH session to use for executing commands.
            _sudo_pwd (str): The password for sudo access.

        Methods:
            __init__(self, ssh_session: SshSession, sudo_paswd: str): Initializes a new instance of the MuttService class.
            start(self): Starts the Mutt service.
            check_env(self): Checks if the environment is set up correctly for the Mutt service.
            make_muttrc_conf(self, hostname, user, paswd): Creates a new muttrc configuration file with the specified parameters.
            set_local_hosts(self, ip, hostname): Sets the local hosts file to include the specified IP and hostname.
            clean_up(self): Cleans up the Mutt service.
    """
    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._sudo_pwd = sudo_paswd

    @root_access_required
    def start(self):
        """
        Starts the Mutt service.
        """
        self._ssh.command('docker exec -dit mutt sh -c "mutt"')

    def check_env(self):
        """
        Checks if the environment is set up correctly for the Mutt service.
        """
        output = self._ssh.command('docker ps -a')
        if "mutt" not in output:
            output = self._ssh.command('docker images | grep alpine')
            if "alpine" not in output:
                self._ssh.command('docker run -dit --name mutt alpine:latest /bin/sh',
                                exact_prompts=['Downloaded'],
                                timeout=120)
            else:
                self._ssh.command('docker run -dit --name mutt alpine:latest /bin/sh')
            time.sleep(3)
            self._ssh.command('docker exec -it mutt apk update',
                            exact_prompts=['OK:'],
                            timeout=120)
            time.sleep(3)
            self._ssh.command('docker exec -it mutt apk add mutt',
                            exact_prompts=['OK:'],
                            timeout=120)
            time.sleep(3)
        else:
             self._ssh.command('docker start mutt',
                            exact_prompts=['\nmutt'],
                            timeout=120)

    def make_muttrc_conf(self, hostname, user, paswd):
        """
        Creates a new muttrc configuration file with the specified parameters.

            Args:
                hostname (str): The hostname of the mail server.
                user (str): The username to use for authentication.
                paswd (str): The password to use for authentication.
        """
        config_content = f"""
set hostname="{hostname}"
set from="{user}@{hostname}"
set realname="{user}"
set imap_user="{user}@{hostname}"
set imap_pass="{paswd}"
set folder="imap://{hostname}:143"
set spoolfile="imap://{hostname}:143/INBOX"
set smtp_url="smtp://{user}@{hostname}@{hostname}:25/"
set smtp_pass="{paswd}"
unset ssl_starttls
unset ssl_force_tls
"""
        escaped_content = config_content.strip().replace("\n", "\\n")
        # 使用printf命令將配置内容寫入~/.muttrc文件
        command = f'''docker exec -it mutt sh -c "printf '{escaped_content}' > ~/.muttrc"'''
        self._ssh.command(command)

    def set_local_hosts(self, ip, hostname):
        """
        Sets the local hosts file to include the specified IP and hostname.

            Args:
                ip (str): The IP address to add to the hosts file.
                hostname (str): The hostname to add to the hosts file.
        """
        output = self._ssh.command(f'''docker exec -it mutt sh -c "cat /etc/hosts"''')
        if hostname not in output:
            self._ssh.command(f'''docker exec -it mutt sh -c "echo '{ip}  {hostname}' >> /etc/hosts"''')
    
    def clean_up(self):
        """
        Cleans up the Mutt service.
        """
        output = self._ssh.command(f'''docker ps | grep mutt''')
        if "mutt" in output:
            self._ssh.command(f'''docker stop mutt && docker rm mutt''', exact_prompts=['\nmutt'], timeout=120)


import re
import sys
import time

from host import *
from behave import *
from steps import common, interface, login


@given('starts tshark sniffer on "{host}"')
@when('starts tshark sniffer on "{host}"')
def step_impl(context, host):
    if context.table:
        context.hosts[host].tshark.start(interface=context.table[0]['iface'], \
                                         capture_filter=context.table[0]['filter'], \
                                         display_filter=context.table[0]['display'])
    else:
        context.hosts[host].tshark.start(interface=context.host_info[host]['nic'])


@when('"{host_send}" send {count} ICMP ping to "{host_recv}"')
def step_impl(context, host_send, count, host_recv):
    context.hosts[host_send].shellcmd.remove_arp_entry(
        dev = context.host_info[host_send]['nic'], \
        ip = context.host_info[host_recv]['testbed']['cur_host'], \
    )
    context.hosts[host_send].hping3.send( \
        tcpudp_flags=['icmp'], \
        host = context.host_info[host_recv]['testbed']['cur_host'], \
        count = count,
        interval = "100000")
    time.sleep(2)


@when('"{host_send}" send {count} ARP probe to "{host_recv}"')
def step_impl(context, host_send, count, host_recv):
    arp_scapy_syntax = f"Ether(\
        dst=\"{context.host_info[host_recv]['testbed']['mac_address']}\", \
        src=\"{context.host_info[host_send]['testbed']['mac_address']}\", \
        type=0x806)"

    context.hosts[host_send].scapy.send(scapy_syntax=arp_scapy_syntax,
                                        dev=context.host_info[host_send]['nic'],
                                        count=int(count))
    time.sleep(2)


@then('"{host_recv}" shall receive {count} ICMP request from "{host_send}"')
def step_impl(context, host_recv, count, host_send):
    context.hosts[host_recv].tshark.stop()

    ret = context.hosts[host_recv].tshark.retrieve()
    print('receive packets: ', ret)

    if len(ret) == 0:
        raise ValueError('packet receiving fails')

    ret_lines = ret.split('\n')

    destip_count = {}
    # parse and get dest IP counts
    for a in ret_lines:
        # skip if not ICMP request
        if a.find("Echo (ping) request") == -1:
            continue

        arr = a.lstrip(' ').split(' ')

        # skip unexpected format
        if arr[3] != '→':
            raise ValueError(f'unexpected tshark format {a}')

        if arr[4] in destip_count:
            destip_count[arr[4]] += 1
        else:
            destip_count[arr[4]] = 1

    # compare dest ip count
    host_recv_ip = context.host_info[host_recv]['testbed']['cur_host']

    if host_recv_ip not in destip_count:
        recv_packet_count = 0
    else:
        recv_packet_count = destip_count[host_recv_ip]

    assert int(recv_packet_count) == int(
        count
    ), f'{host_recv} expect to receive {count} packets but receive {recv_packet_count}'


@then('"{host_recv}" shall receive {count} ARP probe from "{host_send}"')
def step_impl(context, host_recv, count, host_send):
    context.hosts[host_recv].tshark.stop()

    ret = context.hosts[host_recv].tshark.retrieve()

    if len(ret) == 0:
        raise ValueError('packet receiving fails')

    pkt_count = 0

    ret_lines = ret.split('\n')
    for a in ret_lines:
        # skip if not ICMP request
        if a.find("ARP") == -1:
            continue

        arr = a.lstrip(' ').split(' ')

        # skip unexpected format
        if arr[3] != '→':
            raise ValueError(f'unexpected tshark format {a}')

        # Match Source and Destination MAC Address
        if arr[2] == context.host_info[host_send]['testbed'][
                'mac_address'] and arr[4] == context.host_info[host_recv][
                    'testbed']['mac_address']:
            pkt_count += 1

    assert pkt_count == int(
        count
    ), f'{host_recv} expect to receive {count} packets but receive {pkt_count}'


@given('l2-policy install on "{dut}"')
def step_impl(context, dut):
    if context.table[0]['src_mac'] == "all":
        src_mac = "00:00:00:00:00:00"
    elif context.table[0]['src_mac'].find('\'s') != -1:
        host_name = context.table[0]['src_mac'].split('\'')[0]
        src_mac = context.host_info[host_name]["testbed"]["mac_address"]
    else:
        src_mac = context.table[0]['src_mac']

    if context.table[0]['dst_mac'] == "all":
        dst_mac = "00:00:00:00:00:00"
    elif context.table[0]['dst_mac'].find('\'s') != -1:
        host_name = context.table[0]['dst_mac'].split('\'')[0]
        dst_mac = context.host_info[host_name]["testbed"]["mac_address"]
    else:
        dst_mac = context.table[0]['dst_mac']

    if context.table[0]['ethertype'] == "all":
        context.dut[dut].go_config_l2filter(1).\
            action(context.table[0]['action']).\
            protocol("all").\
            src_mac(src_mac).\
            dst_mac(dst_mac).\
            exit()
    else:
        context.dut[dut].go_config_l2filter(1).\
            action(context.table[0]['action']).\
            protocol("manual").\
            ether_type(context.table[0]['ethertype']).\
            src_mac(src_mac).\
            dst_mac(dst_mac).\
            exit()


@when('change "{host}" MAC address, which is different from the original one')
def step_impl(context, host):
    orig_mac = context.host_info[host]["testbed"]["mac_address"]
    orig_mac_arr = orig_mac.split(":")
    another_mac = ""

    # "+1" to current MAC address (wrap around if needed)
    for i in range(0, 6):
        if i != 0:
            another_mac += ":"
        if i == 5:
            another_mac += f'{(int(orig_mac_arr[i], 16)+1)%256:0{2}x}'
        else:
            another_mac += orig_mac_arr[i]

    use_fixture(changeHostMacAndRestore, context, host, another_mac)


@fixture
def changeHostMacAndRestore(context, host_name, mac_address):
    try:
        print("MAC Address Change !!!!!")
        context.hosts[host_name].shellcmd.set_mac_addr(
            dev=context.host_info[host_name]['nic'], mac=mac_address)
        yield
    finally:
        print("MAC Address Restore !!!!!")
        context.hosts[host_name].shellcmd.set_mac_addr(
            dev=context.host_info[host_name]['nic'],
            mac=context.host_info[host_name]["testbed"]["mac_address"])



import os
import subprocess
current_dir = os.path.dirname(__file__)
parent_dir = os.path.dirname(current_dir)

PARENT_DIR = f'{parent_dir}/tn5916_v3_0'
OVERWRITE_LIST = '\|'.join(['bridge'])

print(f'TESTFSM: inheritance the pattern from {PARENT_DIR}')

def get_command_output(command):
    result = subprocess.run(command, shell=True, capture_output=True, text=True)
    output = result.stdout.strip()
    return output
print('file_name:', OVERWRITE_LIST)
cmd_grep = f'ls {PARENT_DIR}| grep -v "{OVERWRITE_LIST}"'
cmd_xargs = f'xargs -I {{}} cp {PARENT_DIR}/{{}} {current_dir}/'
cmd = f'{cmd_grep} | {cmd_xargs}'
print('cmd:', cmd)
print(get_command_output(cmd))


from .router.cli.md_tn5916 import tn5916_v3
from .router.cli.rp_api.rp_api import Router
from .router.cli.md_edrg9010 import edrg9010_v2
from .router.cli.md_edrg9010 import edrg9010_v3

from .router.cli.rp_base.ng_router.base import Base
from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000


from host import *
from behave import *
from steps import login, interface, common
from mdc.router.cli.rp_base.ng_router.base import Base
from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000


@when(u'check Trusted Access default setting on "{device}"')
def step_impl(context, device):
    if issubclass(context.dut[device]._model, tn5000):
        ret = context.dut[device].main().show_trusted_access()
        actual_setting = [ \
        ret[0]["accessible_ip_list"], \
        ret[0]["log"], \
        ret[0]["syslog"], \
        ret[0]["trap"], \
        ret[0]["lan"]]

        expect_setting = [ \
        context.table[0]["Accessible IP List"], \
        context.table[0]["Log Enable"], \
        context.table[0]["Syslog"], \
        context.table[0]["Trap"], \
        context.table[0]["LAN"]]

        assert actual_setting == expect_setting, \
        f'Trusted IP List Setting Mismatch: expect {expect_setting} but {actual_setting}'
    elif issubclass(context.dut[device]._model, Base):
        ret = context.dut[device].main().show_trusted_access()
        actual_setting = [ \
        ret[0]["trusted_access_list"], \
        ret[0]["syslog"], \
        ret[0]["trap"], \
        ret[0]["accept_all_lan"]]

        expect_setting = [ \
        context.table[0]["Trusted Access List"], \
        context.table[0]["Syslog"], \
        context.table[0]["Trap"], \
        context.table[0]["Accept All LAN"]]

        assert actual_setting == expect_setting, \
        f'Trusted IP List Setting Mismatch: expect {expect_setting} but {actual_setting}'


@then(u'"{action}" to access "{device}" with "{protocol}" from "{interface}"')
def step_impl(context, action, protocol, interface, device):
    for row in context.table:
        flag = {'success': True, 'fail': False}[row[action]]
        cmd = {
            "HTTP": {
                "LAN":
                    f'curl -I {context.dut_info[device]["testbed"]["lan_ip"]} --connect-timeout 10',
                "WAN":
                    f'curl -I {context.dut_info[device]["testbed"]["wan_ip"]} --connect-timeout 10'
            },
            "HTTPS": {
                "LAN":
                    f'curl -I https://{context.dut_info[device]["testbed"]["lan_ip"]} --connect-timeout 10',
                "WAN":
                    f'curl -I https://{context.dut_info[device]["testbed"]["wan_ip"]} --connect-timeout 10'
            },
            "Telnet": {
                "LAN":
                    f'telnet {context.dut_info[device]["testbed"]["lan_ip"]}',
                "WAN":
                    f'telnet {context.dut_info[device]["testbed"]["wan_ip"]}'
            },
            "SSH": {
                "LAN":
                    f'ssh admin@{context.dut_info[device]["testbed"]["lan_ip"]}',
                "WAN":
                    f'ssh admin@{context.dut_info[device]["testbed"]["wan_ip"]}'
            },
        }[row['protocol']][row['interface']]
        host = {'WAN': 'HOST_A', 'LAN': 'HOST_B'}[row['interface']]
        print(cmd)
        if flag:
            if row['protocol'] == 'HTTP':
                ret = context.hosts[host].shellcmd._ssh.command(cmd, timeout=5)
                assert any(map(lambda s: s in ret.lower(), \
                               ["http/1.1 200 ok", "http/1.1 308 permanent redirect", "http/1.0 302 redirect"])), \
                                f'[HTTP shall succeed but fail]: ret = {ret}'
            if row['protocol'] == 'HTTPS':
                ret = context.hosts[host].shellcmd._ssh.command(cmd, timeout=5)
                # assert "SSL certificate problem" in ret, f'[HTTPS shall succeed but fail]'
                assert "ssl certificate problem" in ret.lower(), \
                f'[HTTPS shall succeed but fail]: ret = {ret}'
            if row['protocol'] == 'Telnet':
                ret = context.hosts[host].shellcmd._ssh.command(
                    cmd, timeout=5, exact_prompts='login: ')
                context.hosts[host].shellcmd._ssh.sendcontrol('c')
                # assert "Connected" in ret, f'[TELNET shall succeed but fail]'
                assert "connected" in ret.lower(), \
                f'[TELNET shall succeed but fail]: ret = {ret}'
            if row['protocol'] == 'SSH':
                ret = context.hosts[host].shellcmd._ssh.command(cmd,
                                                           timeout=5,
                                                           exact_prompts='password')
                context.hosts[host].shellcmd._ssh.sendcontrol('c')
                # assert ("Password" in ret) or ("RSA key fingerprint" in ret), f'[SSH shall succeed but fail]'
                assert (context.dut_info[device]["testbed"][f"{row['interface'].lower()}_ip"]in ret.lower()) or ("rsa key fingerprint" in ret.lower()), \
                f'[SSH shall succeed but fail]: ret = {ret}'
        else:
            if row['protocol'] == 'HTTP':
                ret = context.hosts[host].shellcmd._ssh.command(cmd, timeout=5)
                # assert (ret == None) or ("HTTP/1.1 200 OK" not in ret), f'[HTTP shall fail but succeed]'
                assert (ret == None) or ("http/1.1 200 OK" not in ret.lower()) or ("http/1.1 308 permanent redirect" in ret.lower()), \
                f'[HTTP shall fail but succeed]: ret = {ret}'
            if row['protocol'] == 'HTTPS':
                ret = context.hosts[host].shellcmd._ssh.command(cmd, timeout=5)
                # assert (ret == None) or "SSL certificate problem" not in ret, f'[HTTPS shall fail but succeed]'
                assert (ret == None) or "ssl certificate problem" not in ret.lower(), \
                f'[HTTPS shall fail but succeed]: ret = {ret}'
            if row['protocol'] == 'Telnet':
                ret = context.hosts[host].shellcmd._ssh.command(
                    cmd, timeout=5, exact_prompts='login: ')
                context.hosts[host].shellcmd._ssh.sendcontrol('c')
                # assert (ret == None) or ("Connected" not in ret), f'ret= {ret}, [TELNET shall fail but succeed]'
                assert (ret == None) or ("connected" not in ret), \
                f'ret= {ret}, [TELNET shall fail but succeed]: ret = {ret}'
            if row['protocol'] == 'SSH':
                ret = context.hosts[host].shellcmd._ssh.command(cmd,
                                                           timeout=5,
                                                           exact_prompts=':')
                context.hosts[host].shellcmd._ssh.sendcontrol('c')
                # assert (ret == None) or ("Password" in ret) or ("RSA key fingerprint?" not in ret), f'ret= {ret}, [SSH shall fail but succeed]'
                assert (ret == None) or ("password" in ret.lower()) or ("rsa key fingerprint?" not in ret.lower()), \
                f'[SSH shall fail but succeed]: ret = {ret}'



from mdc.router.cli.rp_base.ng_router import config
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigL2filter(config.Config):
    """
    Class for configuring L2 filter.

        Args:
            session (object): A valid session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session: object, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
        Set the default prompts.
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    # Reserve flexibility for function override
    def _getin(self):
        """
        Get into the interface lan.
        """
        # default apply the first element as parameter
        policy_index = self._args[0] if len(self._args) != 0 else None
        if policy_index is None:
            raise Exception(f"Input parameter is empty!")
        self._s.command_expect(f'l2-filter {policy_index}')

    def getin(self):
        """
        Set the default prompts and get into the interface wan.

            Returns: 
                self: The instance of the object. 
        """
        self._set_default_prompts()
        self._getin()
        return self

    def action(self, action: str):
        """
        Set the action of the rule.

            Args:
                action (str): The action of the rule. Can be either "accept" or "drop". 
            
            Returns: 
                self: The instance of the object. 
        """
        action_list = ["accept", "drop"]

        if action in action_list:
            self._s.command_expect(f'action {action}')
        else:
            raise ValueError('action can be either accept or drop')

        return self

    def protocol(self, protocol: str = "all"):
        """
        Set the protocol of the rule. 

            Args: 
                protocol (str): The protocol of the rule. Default is "all". 

            Returns: 
                self: The instance of the object. 
        """
        self._s.command_expect(f'protocol {protocol}')

        return self

    def ether_type(self, ether_type: str):
        """
        Set the ether type of the rule. 

            Args: 
                ether_type (str): The ether type of the rule.  

            Returns: 
                self: The instance of the object.
        """
        self._s.command_expect(f'ether-type {ether_type}')
        return self

    def src_mac(self, src_mac: str):
        """
        Set source mac address of the rule.  

            Args:  
                src_mac (str): Source mac address to set for this rule.  

            Returns:  
                self: The instance of the object.
        """
        self._s.command_expect(f'src-mac {src_mac}')
        return self

    def dst_mac(self, dst_mac: str):
        """
        Set the destination MAC address of the rule.

            Args:
                dst_mac (str): The destination MAC address.

            Returns:
                self: The instance of the object.
        """
        self._s.command_expect(f'dst-mac {dst_mac}')
        return self

    def interface(self, if_from: str = "all", if_to: str = "all"):
        """
        Set the interfaces of the rule.

            Args:
                if_from (str, optional): The starting interface. Defaults to "all".
                if_to (str, optional): The ending interface. Defaults to "all".

            Returns:
                self: The instance of the object. 
        """
        self._s.command_expect(f'interface {if_from} {if_to}')
        return self

    def exit(self):
        """
        Exit the rule setting

                Returns:
                self: The instance of the object. 
        """
        ret = self._s.command_expect('exit',
                                        prompts=[PROMPT_CONFIG],
                                        timeout=2)
        return self



import re
import sys
import time

from host import *
from behave import *
from steps import common, interface, login


@when('"{device}" {dosType} Dos Defense turns {flag}')
def step_impl(context, flag, dosType, device):
    switch = {'on': True, 'off': False}[flag.lower()]

    if switch:
        context.dut[device].go_config().command(f'dos {dosType.lower()}-scan')
    else:
        context.dut[device].go_config().command(
            f'no dos {dosType.lower()}-scan')

    dos_status = context.dut[device].main().show_dos()
    expect = {'on': 'Enable', 'off': 'Disable'}[flag.lower()]
    assert expect == dos_status[0]['syn_rst_scan'].strip(), \
           f'show_dos syn/rst-scan is not {expect}: {dos_status[0]["syn_rst_scan"]}'


@when('"{device}" Dos Defense {logType} log turns {flag}')
def step_impl(context, flag, logType, device):
    switch = {'on': True, 'off': False}[flag.lower()]
    if flag:
        context.dut[device].go_config().command(f'logging dos')
        context.dut[device].go_config().command(f'logging dos {logType}')
    else:
        context.dut[device].go_config().command(f'no logging dos {logType}')

    dos_status = context.dut[device].main().show_dos()
    expect = {'on': 'Enable', 'off': 'Disable'}[flag.lower()]

    assert expect == dos_status[0]['flash'].strip(), \
           f'show_dos Flash is not {expect}: {dos_status[0]["flash"]}'


@when('"{host}" starts tshark to sniffer')
def step_impl(context, host):
    context.hosts[host].tshark.start()


@when('"{host}" waits {second} seconds then stops tshark snifferring')
def step_impl(context, second, host):
    time.sleep(int(second))
    context.hosts[host].tshark.stop()


@when(
    '"{host1}" sends {pack_num} TCP packets in flag of SYN and RST to "{host2}"\'s port {port}'
)
def step_impl(context, pack_num, port, host1, host2):
    # sudo hping3 192.168.127.254 -R -S --faster -p 80 -c 1000
    dst = context.host_info[host2]['testbed']['cur_host']
    context.hosts[host1].hping3.send(host=dst,
                                     port=port,
                                     tcpudp_flags=['syn', 'rst'],
                                     count=pack_num)


@then('"{host1}" received {pack_num} packets from "{host2}"')
def step_impl(context, pack_num, host1, host2):
    result = context.hosts[host1].tshark.retrieve()
    # print(result)
    expect = int(pack_num) * 0.9
    pattern = f"{context.host_info[host2]['testbed']['cur_host']} → " \
              f"{context.host_info[host1]['testbed']['cur_host']}"
    count = result.count(pattern)
    print("%" * 20, f'verify syn-rst Tshark count: {count}', "%" * 20)
    if expect == 0:
        assert count == expect, f'expect {expect} but get {count}'
    else:
        assert count >= expect, f'expect {expect} but get {count}'


@then(
    '"{device}" logging {detect} attack events from "{host1}" to "{host2}" in the period of testing'
)
def step_impl(context, detect, device, host1, host2):
    verifier = {'some': lambda c: c > 1, 'no': lambda c: c == 0}[detect.lower()]

    count = 0
    logs = context.dut[device].main().show_logging_event_log_dos()
    expect_log = (context.host_info[host1]['testbed']['cur_host'], \
        context.host_info[host2]['testbed']['cur_host'], \
            'TCP', \
            'DROP')
    for log in logs:
        # print(log['src_ip'], log['dest_ip'], log['protocol'], log['action'])
        if (log['src_ip'], log['dest_ip'], log['protocol'],
                log['action']) == expect_log:
            count += 1
    print("%" * 20, f'verify Event Log count: {count}', "%" * 20)
    assert verifier(count), f'expect {detect} but get {count}'



import os
import time

def telnet_login(session: object,
                 username: str,
                 password: str,
                 prompts_main: list,
                 prompts_config: list,
                 login_prompt: str = 'login'):
    """
    Log in to a telnet session with given username and password.

        Args:
            session (TelnetSession): Telnet session object.
            username (str): Username for login.
            password (str): Password for login.
            prompts (list): Prompts to expect after login.
            login_prompt (str, optional): Login prompt string, defaults to 'login: '. 
            
        Raises:
            ValueError: If cannot find login prompt or prompt after password. 
    """
    quick_timeout = 2

    # if login session existed, send enter to let prompt appear
    # print('try to get main prompt') #debug
    ret = session.command_expect('', prompts=prompts_main, timeout=quick_timeout)
    # consume prompts to make buffer clean
    # ret = session.command_expect(None, prompts=promtps, timeout=quick_timeout)
    if ret['matched']:
        return
    
    # print('try to get config prompt') #debug
    # if login session is config prompt, try to back to main prompt
    ret = session.command_expect('', prompts=prompts_config, timeout=quick_timeout)
    if ret['matched']:
        # print('config mode') #debug
        is_main_prompt = False
        for i in range(120):
            # print('try to login') #debug
            time.sleep(1)
            is_main_prompt = session.command_expect('exit', prompts=prompts_main, timeout=quick_timeout)['matched']
            if is_main_prompt:
                break
            # print('exit to main prompt') #debug
        return

    # try to press key to pass "Escape character is '^]'." if it occours
    # print('try to find login prompt 1') #debug
    ret = session.command_expect('',
                                 prompts=[login_prompt],
                                 exact_str=True,
                                 timeout=quick_timeout)
    if not ret['matched']:
        # print('try to find login prompt 2') #debug
        ret = session.command_expect('',
                                     prompts=[login_prompt],
                                     exact_str=True,
                                     timeout=quick_timeout)

    if not ret['matched']:
        raise ValueError(f'cannot find login prompt: "{login_prompt}" in {ret}')
    # print(f'input username: {username}') #debug
    session.command_expect(username,
                           prompts=['Password: '],
                           exact_str=True,
                           timeout=quick_timeout)
    # print(f'input password: {password}') #debug
    ret = session.command_expect(password, timeout=quick_timeout)
    if not ret['matched']:
        raise ValueError(f'cannot find prompt after password: {ret}')


def reload_factory_default(session: object,
                           prompts=[
                               'Proceed with reload to factory default? [Y/n]'
                           ]):
    """
    Reload the device to the factory default settings. 
        
        Args: 
            session (object): The session object for the device. 
            prompts (list): A list of strings that are expected from the device. 
    
    """
    session.command_expect('reload factory-default',
                           prompts=prompts,
                           exact_str=True)
    session.command_expect('Y')


def reload_factory_default_no_cert(session: object,
                           prompts=[
                               'Proceed with reload to factory default? [Y/n]'
                           ]):
    """
    Reload the device to the factory default settings.
    With no "Certificate Management" and "Authentication Certificate" configuration.
        
        Args: 
            session (object): The session object for the device. 
            prompts (list): A list of strings that are expected from the device. 
    
    """
    session.command_expect('reload factory-default no cert',
                           prompts=prompts,
                           exact_str=True)
    session.command_expect('Y')


def reload(session: object, prompts: list = ['Proceed with reload ? [Y/n]']):
    """
    Reload the device. 
        Args: 
            session (object): The session object for the device. 
            prompts (list): A list of strings that are expected from the device. 
    """
    session.command_expect('reload', prompts=prompts, exact_str=True)
    session.command_expect('Y')


def login(session: object,
          prompts1: list = ['Password:'],
          prompts2: list = ['#']):
    """
    Login to the device using admin credentials. 
        Args: 
            session (object): The session object for the device.  
            prompts1 (list): A list of strings that are expected from the device after entering 'admin'.  
            prompts2 (list): A list of strings that are expected from the device after entering 'moxa'.
    """
    session.command_expect('admin', prompts=prompts1, exact_str=True)
    session.command_expect('moxa', prompts=prompts2, exact_str=True)


import re

IPV4_REGEXP = r'^([1-9]|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])(\.(0|[1-9]|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])){3}$'
MAC_REGEXP = r'^[a-fA-F0-9]{2}(?:[:-]?[a-fA-F0-9]{2}){5}(?:,[a-fA-F0-9]{2}(?:[:-]?[a-fA-F0-9]{2}){5})*$'


def chk_valid_ip(ip):
    rtv4 = None
    rtv6 = None
    try:
        chk_valid_ipv4(ip)
    except:
        rtv4 = False
    else:
        rtv4 = True
    try:
        chk_valid_ipv6(ip)
    except:
        rtv6 = False
    else:
        rtv6 = True
    if (rtv4 | rtv6) is False:
        raise ValueError('Invalid IP address')


def chk_valid_ipv4(ipv4):
    if re.search(IPV4_REGEXP, ipv4) is None:
        raise ValueError('Invalid IPv4 address')


def chk_valid_ipv6(ipv6):
    raise ValueError('No implementation yet')


def chk_is_private_ip(ip) -> bool:
    return chk_is_private_ipv4(ip) or chk_is_private_ipv6(ip)


def chk_is_private_ipv4(ipv4) -> bool:
    try:
        chk_valid_ipv4(ipv4)
    except:
        return False
    """
    Private IP range: 
    Class A: 10.0.0.0 ~ 10.255.255.255
    Class B: 172.16.0.0 ~ 172.31.255.255
    Class C: 192.168.0.0 ~ 192.168.255.255
    """
    ipv4_parted = ipv4.split('.')
    if int(ipv4_parted[0]) == 10:
        # Class A
        return True
    elif (int(ipv4_parted[0]) == 172) and (int(ipv4_parted[1]) >= 16 and
                                           int(ipv4_parted[1]) <= 31):
        # Class B
        return True
    elif int(ipv4_parted[0]) == 192 and int(ipv4_parted[1]) == 168:
        # Class C
        return True
    else:
        # non-private IP
        return False


def chk_is_private_ipv6(ipv6) -> bool:
    try:
        chk_valid_ipv6(ipv6)
    except:
        return False

    # TODO
    """
    Private IP block: 
    fd00::/8
    """


def chk_valid_mac(mac_addr):
    if re.search(MAC_REGEXP, mac_addr) is None:
        raise ValueError('Invalid MAC address')



from utils.input_chk import chk_valid_ip
from mdc.router.cli.rp_base.lg_router.tn5000 import cli
from mdc.router.cli.rp_base.lg_router.tn5000.cli import PROMPT_CONFIG

class Config(cli.Cli):
    """
    Class for configuring a device through the command line interface.

        Args:
            session (Session): A Session object used to communicate with the device.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
    """

    def __init__(self, session):
        super().__init__(session)
        # back to main
        self._back_to_main()
        # jumpt to config terminal
        self.getin_cfg()

    def get_session(self):
        return self._s

    def _set_default_prompts_cfg(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin_cfg(self):
        self._s.command_expect('configure terminal')

    # function override is allowed
    def getin_cfg(self):
        self._set_default_prompts_cfg()
        self._getin_cfg()
        return self

    # ======== dhcp feature ========
    class DhcpService:

        def __init__(self, session):
            self._s = session

        def set_global_dhcp_service(self,
                                    action: str = "enable",
                                    mode: str = ""):
            """
            Configure global DHCP service.
            
                Args
                    mode: global dhcp mode ("" or auto-assign).
                    action: global dhcp action (enable or disable).
            """
            cmd = "service dhcp"
            if action == "enable":
                cmd = f"{cmd} {mode}"
            elif action == "disable":
                cmd = f"no {cmd} {mode}"
            else:
                raise ValueError(f"input invalid action: {action}")

            self._s.command_expect(cmd)

        def set_global_dhcp_server(self,
                                    mode: str = "pool",
                                    action: str = "enable",
                                    index: str = "1",
                                    name: str = "dhcp_server"):
            '''
            Args
                mode: dhcp server mode (pool or static pool).
                name: dhcp server name. Defaults to "dhcp_server".
                index: dhcp server index. Defaults to "1".
                action: dhcp server action (enable or disable).
            '''
            cmd = "ip dhcp"
            if mode == "pool":
                cmd = f"{cmd} {mode} {index} {action}"
            elif mode == "static pool":
                cmd = f"{cmd} {mode} {name} {action}"
            else:
                raise ValueError(f"input invalid dhcp mode: {mode}")

            self._s.command_expect(cmd)

    # ======== End of dhcp feature ========
    # ======== login lockout feature ========
    class LoginLockout:

        def __init__(self, session):
            self._s = session
            self._cmd = 'login-lockout'

        def lockout_toggle(self, toggle):
            if toggle is True:
                self._s.command_expect(f'{self._cmd}')
            else:
                self._s.command_expect(f'no {self._cmd}')

        def lockout_retryThreshold(self, retry):
            self._s.command_expect(f'{self._cmd} retry-threshold {retry}')

        def lockout_lockoutTime(self, duration):
            self._s.command_expect(f'{self._cmd} lockout-time {duration}')

    # ======== End of login lockout feature ========
    # ======== System Information feature ========
    class system_information:

        def __init__(self, session):
            self._s = session

    # ======== End of System Information feature ========
    # ======== time sync feature ========
    class TimeSync:

        def __init__(self, session):
            self._s = session

        def clock_source(self, clk_src, remote_server=None):
            if clk_src.lower() == "local":
                self._s.command_expect(f'no ntp remote-server')
            elif clk_src.lower() == "ntp" or clk_src.lower() == "sntp":
                chk_valid_ip(remote_server)
                cmd = f'ntp remote-server {remote_server}'
                if clk_src.lower() == "sntp":
                    cmd += ' simple'
                self._s.command_expect(cmd)
            else:
                raise Exception(f"Unsupported Clock Source: {clk_src}")

        def ntp_server_toggle(self, toggle):
            if toggle is True:
                self._s.command_expect(f'ntp server')
            else:
                self._s.command_expect(f'no ntp server')

        def set_time_manually(self, year, month, day, hour, min, sec):
            self._s.command_expect(
                f'clock set {hour}:{min}:{sec} {month} {day} {year}')

    # ======== End of time sync feature ========

    def set_clock_summer_time(self,
                                start_date: str = None,
                                end_date: str = None,
                                offset: str = None):
        """
        Set the Daylight Saving for the device.
            Args: 
                start_date (str): The Date when Summer Time Offset Start. The format should be "{month} {week} {day} {hour} {min}" 
                end_date (str): The Date when Summer Time Offset End. The format should be "{month} {week} {day} {hour} {min}"
                offset (str): Summer Time Offset. (1 - 12)
            Explanation of the Time format:
                month: From 'Jan', 'January' or '1' to 'Dec', 'December', or '12'
                week: From '1st' or '1' to 'Last' or '6'
                day: From 'Sun', 'Sunday' or '1' to 'Sat', 'Saturday' or '7'
                hour: 0 - 23
                min: 0 - 59
        """
        if start_date:
            self._s.command_expect(
                f'clock summer-time start-date {start_date}')
        if end_date:
            self._s.command_expect(f'clock summer-time end-date {end_date}')
        if offset:
            self._s.command_expect(f'clock summer-time offset {offset}')

    def set_hostname(self, hostname):
        """
        Sets the hostname of the device.

            Args:
                hostname (str): The name of the device. 
        """
        self._s.command_expect(command=f'hostname {hostname}')

    def set_login_account(self, action, username, password, privilege=None):
        cmd = ""
        if username:
            cmd += f"username {username} "
        if password:
            cmd += f"password {password} "

        if privilege != None:
            if privilege == "admin":
                cmd += "privilege 1"
            elif privilege == "user":
                cmd += "privilege 2"
            elif privilege == "no login":
                cmd += "privilege 3"
            else:
                raise ValueError(f'input invalid privilege: {privilege}')

        if action != None:
            if action == "delete":
                cmd = f"no {cmd}"
            elif action in ["create", "modify"]:
                pass
            else:
                raise ValueError(f'input invalid action: {action}')
        return self._s.command_expect(cmd, prompts=["#"], exact_str=True)

    def set_password_policy(self,
                            min_len=None,
                            action=None,
                            global_complexity_check=None,
                            complexity_item=None):
        cmd = "password-policy "
        if min_len:
            cmd += f"minimum-length {min_len}"
        elif global_complexity_check:
            cmd += f"complexity-check"

        if complexity_item != None:
            if complexity_item == "digit":
                cmd += f"complexity-check digit"
            elif complexity_item == "alphabet":
                cmd += f"complexity-check alphabet"
            elif complexity_item == "special-characters":
                cmd += f"complexity-check special-characters"
            else:
                raise ValueError(
                    f'input invalid complexity item: {complexity_item}')

        if action != None:
            if action == "disable":  # can only disable complexity
                cmd = f"no {cmd}"
            elif action == "enable":
                pass
            else:
                raise ValueError(f'input invalid action: {action}')

        return self._s.command_expect(cmd, prompts=["#"], exact_str=True)

    def set_log(self, log_item=None):
        log_list = ["trusted-access", "dos", "firewall"]
        if log_item in log_list:
            self._s.command_expect(f"logging {log_item}",
                                    prompts=[PROMPT_CONFIG])
        else:
            raise ValueError(f'input invalid log item: {log_item}')
        return self

    def set_syslog_server(self, action, ip, port=514):
        if action == "create":  # create server will enable itself as well
            ret = self._s.command_expect(f"logging {ip} {port}",
                                            prompts=['#'])
            if "Server list is full." in ret['data']:
                raise ValueError(f'Server list is full.')
        elif action in ["enable", "disable"]:
            self._s.command_expect(f"logging {ip} {port} {action}",
                                    prompts=[PROMPT_CONFIG])
        else:
            raise ValueError(f'input invalid action: {action}')

    def set_warning_notification(self,
                                    event,
                                    action=None,
                                    severity=None,
                                    active=None):
        action_type = {
            "Trap only": 1,
            "Email only": 2,
            "Trap+Email": 3,
            "Syslog only": 4,
            "Trap+Syslog": 5,
            "Email+Syslog": 6,
            "Trap+Email+Syslog": 7,
            "Relay1 only": 8,
            "Trap+Relay1": 9,
            "Email+Relay1": 10,
            "Trap+Email+Relay1": 11,
            "Syslog+Relay1": 12,
            "Trap+Syslog+Relay1": 13,
            "Email+Syslog+Relay1": 14,
            "Trap+Email+Syslog+Relay1": 15
        }  # the list goes on...
        event_type = [
            "cold-start", "warm-start", "pwr1-trans-on", "pwr2-trans-on",
            "pwr1-trans-off", "pwr2-trans-off", "auth-fail",
            "config-changed"
        ]
        severity_type = {
            "Emergency": 0,
            "Alert": 1,
            "Critical": 2,
            "Error": 3,
            "Warning": 4,
            "Notice": 5,
            "Information": 6,
            "Debug": 7
        }
        if event in event_type:
            if active == True:
                self.command(
                    f"warning-notification system-event {event} active")
            if action in action_type:
                self.command(
                    f"warning-notification system-event {event} action {action_type[action]}"
                )
            if severity in severity_type:
                self.command(
                    f"warning-notification system-event {event} severity {severity_type[severity]}"
                )
        else:
            raise ValueError(f'input invalid event: {event}')

    def set_trusted_access(self,
                            ip=None,
                            mask=None,
                            action=None,
                            global_toggle=None):
        if global_toggle:
            if global_toggle == "enable":
                self.command(f"interface trusted-access")
            elif global_toggle == "disable":
                self.command(f"no interface trusted-access")
            else:
                raise ValueError(
                    f'input invalid global_toggle: {global_toggle}')
        if ip and mask:
            if action == "enable":
                self.command(f"interface trusted-access {ip} {mask}")
            elif action == "delete":
                self.command(f"no interface trusted-access {ip} {mask}")
            else:
                raise ValueError(f'input invalid action: {action}')

    def set_vlan_id(self, vlan_id):
        self._s.command_expect(f"vlan create {vlan_id}")
        return self

    def set_ip_ping_response(self):
        self._s.command_expect("ip ping-response")
        return self

    def set_ip_http_max_login_users(self, max_users):
        self._s.command_expect(
            f"ip http-server max-login-users {max_users}")
        return self

    def set_global_telnet(self, port=None, max_login_users=None):
        if port != None:
            self._s.command_expect(f"ip telnet port {port}")
        if max_login_users != None:
            self._s.command_expect(
                f"ip telnet max-login-users {max_login_users}")

    def set_firewall_rule_status(self, context, index, status):
        """
        Modify the status of a specified firewall rule.

        Args:
            index (int): The index of the firewall rule to modify.
            status (str): The status to set for the firewall rule. "enable" to enable the rule, "disable" to disable it.
        """
        self._s.command_expect(f"firewall {index} {status}")
        return self

    def set_global_vrrp(self, version: str = "3", status: str = "enable"):
        """
         Sets the VRRP status of the device. This is useful for debugging and to make it easier to use the device in a real - time program
         
         Args:
         	 version: The version of the RTP device
         	 status: The status of the RTP device ( enable|disable )
         
         Returns: 
         	 The self for method chaining ( self. _s )
        """
        # Set vrrp version
        if version:
            self._s.command_expect(f"vrrp version {version}")
        # Enable or disable the vrrp
        if status == "enable":
            self._s.command_expect(f"router vrrp")
            self._s.command_expect(f"vrrp 1 enable")
        else:
            raise ValueError(f"Input invalid status: {status}")
        return self

    def set_port_mirror(self, src: str = None, dst: str = None):
        """
        Configure Port mirror on a device.

            Args:
                src (str): monitorPort ( Port ID. Ex. 1/3, Trk2)
                dst (str): mirrorPort (Port ID. Ex. 1/3, 2/1)
            Returns: 
                self for daisychaining ( self )
        """
        if src:
            self._s.command_expect(f"monitor source interface {src}")
        if dst:
            self._s.command_expect(f"monitor destination interface {dst}")
        return self


    class MulticastRoute:
        """Class to configure Multicast Route.

            Args:
                session (object): An instance of the Session class.
        """

        def __init__(self, session: object):
            self._s = session

        def set_global_multicast_route(self, mode: str = "static"):
            """
            Configure global multicast route.
            
                Args
                    action: global multicast route mode (static or disable).
            """
            if mode == "static":
                cmd = f"ip multicast-routing static"
            elif mode == "disable":
                cmd = "no ip multicast-routing"
            else:
                raise ValueError(f"input invalid mode: {mode}")

            self._s.command_expect(cmd)

        def set_static_multicast_route_rule(self,
                                            action: str = "add",
                                            group_addr: str = None,
                                            src_addr: str = None,
                                            in_itf: str = None,
                                            out_itf: str = None):
            """
            Configure global DHCP server.

            Args
                action: configure static multicast route (add / enable / disable).
                group_addr: multicast group
                src_addr: multicast stream source
                in_itf: Inbound interface
                out_itf: Outbound interface
            """
            if action == "add":
                cmd = f"ip mroute group {group_addr} src {src_addr} in \
                            {in_itf} out {out_itf}"

            elif action == "enable":
                cmd = f"ip mroute group {group_addr} src {src_addr} enable"
            elif action == "disable":
                cmd = f"ip mroute group {group_addr} src {src_addr} disable"
            else:
                raise ValueError(f"input invalid action: {mode}")

            self._s.command_expect(cmd)

    class IgmpSnooping:
        """Class to configure Igmp Snooping.

            Args:
                session (object): An instance of the Session class.
        """

        def __init__(self, session: object):
            self._s = session

        def set_igmp_snooping_vlan(self, action: str, vid: str):
            """
             Enable or disable snooping for a VLAN.
             
             Args:
             	 action: 'enable'or'disable'.
             	 vid: VLAN ID.
            """
            if action == 'enable':
                self._s.command_expect(f"ip igmp-snooping vlan {vid}")
            elif action == 'disable':
                self._s.command_expect(f"no ip igmp-snooping vlan {vid}")
            else:
                raise ValueError(f'input invalid action')
            return self

        def set_igmp_snooping_querier(self, action: str, vid: str):
            """
             Enable or disable igmp snooping querier on a VLAN.
             
             Args:
             	 action: 'enable'or'disable'.
             	 vid: VLAN ID.
            """
            if action == 'enable':
                self._s.command_expect(
                    f"ip igmp-snooping querier vlan {vid}")
            elif action == 'disable':
                self._s.command_expect(
                    f"no ip igmp-snooping querier vlan {vid}")
            else:
                raise ValueError(f'input invalid action')
            return self

        def set_igmp_snooping_interval(self, interval: str):
            """
             Set IGMP snooping interval. Valid values are 20 - 600 seconds.
             
             Args:
             	 interval: interval in seconds between IGMP queries.
            """
            if 20 <= int(interval) <= 600:
                self._s.command_expect(
                    f"ip igmp-snooping query-interval {interval}")
            else:
                raise ValueError(f'input interval out of range')
            return self

        def set_global_igmp_snooping(self, action: str):
            """
             Enable / Disables global igmp snooping.
             
             Args:
             	 action: 'enable'or'disable'.
            """
            if action == 'enable':
                self._s.command_expect(f"ip igmp-snooping")
            elif action == 'disable':
                self._s.command_expect(f"no ip igmp-snooping")
            else:
                raise ValueError(f'input invalid action')
            return self

    class Rstp:
        def __init__(self, session: object):
            """
             Initialize the instance. This is called by __init__ and should not be called directly
             
             Args:
                 session: The session to use
            """
            self._s = session
        def set_rstp_status(self, action):
            """
             Enable or disable RSTP. Redundancy mode can be enabled or disabled at the same time
             
             Args:
                 action: " enable " or " disable
            """
            # enable disable or rstp.
            if action == "enable":
                self._s.command_expect('redundancy mode rstp')
            # either "spanning tree" or "turbo ring" must be enabled and cannot be enabled or disabled at the same time
            elif action == "disable":
                self._s.command_expect('redundancy mode turbo-ring-v2')
            else:
                raise ValueError(f"input invalid action: {action}")
        
        def set_rstp_port(self, 
                             port,
                             action: str = "enable", 
                             edge_port: bool = False, 
                             priority: str = None, 
                             cost: str = None
                             ):
            """
            Set RSTP settings for a port. This is useful for port selection and retransmission of packets.
            
                Args:
                    port: port to set RSTP settings for.
                    edge_port: if True set RSTP edge settings for this port
                    priority: priority of the packet ( default : 0 )
                    cost: cost of the packet ( default : None )
                
                Returns: 
                    ` ` True ` ` if successful ` ` False ` `
            """
            self._s.command_expect(f'interface ethernet 1/{port}')
            if action == "enable":
                self._s.command_expect(f'spanning-tree')
            elif action == "disable":
                self._s.command_expect(f'no spanning-tree')
            else:
                raise ValueError(f"input invalid status: {action}")
            # expects panning tree edge port command
            if edge_port:
                self._s.command_expect(f'spanning-tree edge-port')
            # expect priority priority if priority is set
            if priority:
                self._s.command_expect(f'spanning-tree priority {priority}')
            # expect the panning tree cost
            if cost:
                self._s.command_expect(f'spanning-tree cost {cost}')
            return self

    def set_static_multicast_table(self, mac: str, port: str = None, vlan: str = None):
        """
        Configure static multicast table on a device

            Args:
                mac (str): Source MAC address.
                port (str): The port use for multicast group. Defaults to None.
                vlan (str): Invalid for TN-5916.
        """
        self._s.command_expect(f'ip igmp static-group {mac} interface 1/{port}')
        return self   


from mdc.router.cli.rp_base.ng_router import config
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigVrrp(config.Config):
    def __init__(self, session: object, *args, **kwargs):
        """
         Initialize the instance. This is called by the : class : ` Session ` when it is created.
         
            Args:
                session: The session to use for the request / response
        """
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
         Set the default prompts for this service. This is called by __init__ and should not be called by user code
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    # Reserve flexibility for function override
    def _getin(self):
        """
         Get in vrrp index from command line. @param self : class object. @return None
        """
        # default apply the first element as parameter
        vrrp_index = self._args[0] if len(self._args) != 0 else None
        # Check if vrrp_index is not None.
        if vrrp_index is None:
            raise Exception(f"Need vrrp index!")
        self._s.command_expect(f'vrrp {vrrp_index}')

    def getin(self):
        """
         Set prompts to getin. This is a shortcut for _set_default_prompts ( self. _default_prompts )
         
         
            Returns: 
                self for daisychaining ( self )
        """
        self._set_default_prompts()
        self._getin()
        return self

    def set_vrrp_rule(self,
                      status: str = None,
                      vrid: str = None,
                      vip: str = None,
                      priority: str = None,
                      preempt_mode: str = None,
                      preempt_delay: str = None,
                      accept_mode: str = None,
                      interface: str = None,
                      adver_interval: str = None,
                      adver_interval_ver: str = "v3"
                      ):
        """
        Set VRRP rule. This is a helper method to allow you to set a rule in the configuration file.
        
            Args:
                status: The status of the rule. Valid values are enabled disabled and un - enabled.
                vrid: The ID of the Virtual Router to set the rule to.
                vip: The IP address of the Virtual Router to set the rule to.
                priority: The priority of the rule. Valid values are 1 - 4.
                preempt_mode: The preempt mode of the rule. Valid values are enabled and disabled.
                accept_mode: The accept mode of the rule. Valid values are enabled and disabled.
                interface: The interface for the rule. Valid values are " eth0 " " eth1 " and " ethernet ".
                adver_interval: The advertisement interval in seconds. Default is 1 second.
                adver_interval_ver: The advertising interval version in seconds. Default is v3.
            
            Returns: 
                self for daisychaining ( self )
        """
                      
        # expects vrid to be passed to the command
        if vrid:
            self._s.command_expect(f'vrid {vrid}')
        # expect virtual ip if vip is given
        if vip:
            self._s.command_expect(f'virtual-ip {vip}')
        # Expects the command to expect priority
        if priority:
            self._s.command_expect(f'priority {priority}')
        # Enable or disable preempting the command.
        if preempt_mode:
            # Enable or disable preempting the command
            if preempt_mode == "enable":
                if preempt_delay:
                    self._s.command_expect(f'preempt delay {preempt_delay}')
                else:
                    self._s.command_expect(f'preempt')
            elif preempt_mode == "disable":
                self._s.command_expect(f'no preempt')
            else:
                raise ValueError(f"Input invalid preempt status: {preempt_mode}")
        # accept_mode is one of enable disable or enable
        if accept_mode:
            # Accept mode enable disable or enable
            if accept_mode == "enable":
                self._s.command_expect(f'accept')
            elif accept_mode == "disable":
                self._s.command_expect(f'no accept')
            else:
                raise ValueError(f"Input invalid accept status: {accept_mode}")
        # expect interface interface interface interface.
        if interface:
            self._s.command_expect(f'interface {interface}')
        # expects the command to be called at the adver interval
        if adver_interval:
            self._s.command_expect(f'adver-interval v{adver_interval_ver} {adver_interval}')
        # Enable or disable the command.
        if status:
            # Enable or disable the command.
            if status == "enable":
                self._s.command_expect(f'vrrp')
            elif status == "disable":
                self._s.command_expect(f'no vrrp')
            else:
                raise ValueError(f"Input invalid status: {status}")
        return self



from mdc.router.cli.rp_base.lg_router.edr800 import config
from mdc.router.cli.rp_base.lg_router.edr800.cli import PROMPT_CONFIG

class ConfigIfVlan(config.Config):
    """
    Class for configuring a VLAN interface.

        Args:
            session (Session): An instance of a Session object.
            *args (list): A list of arguments to be passed to the function.
            **kwargs (dict): A dictionary of keyword arguments to be passed to the function.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    # Reserve flexibility for function override
    def _getin(self):
        # default apply the first element as parameter
        vlan_id = self._args[0] if len(self._args) != 0 else None
        if vlan_id is None:
            raise Exception(f"Input parameter is empty!")
        self._s.command_expect(f'interface vlan {vlan_id}')

    def getin(self):
        self._set_default_prompts()
        self._getin()
        return self

    def set_ip(self, ip, mask):
        self._s.command_expect(f'ip address {ip} {mask}')
        return self

    def set_name(self, name):
        self._s.command_expect(f'name {name}')
        return self



from mdc.router.cli.rp_base.ng_router import config
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigObjectAddr(config.Config):
    """
    Class for configuring IP Address and Subnet Object.

        Args:
            session (object): The session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session: object, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
        Set the default prompts.
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        """
        Get into the interface lan.
        """
        self._s.command_expect(f'object address')

    def getin(self):
        """
        Set the default prompts and get into the interface wan.

            Returns: 
                self: The instance of the object. 
        """
        self._set_default_prompts()
        self._getin()
        return self

    def create_object(self, name: str = None, ip_addr: str = None):
        """
        Create an object with the given name and IP address.

            Args:
                name (str, optional): The name of the object to create.
                ip_addr (str, optional): The IP address of the object to create.
                
            Returns:
                self: The instance of the object. 
        """
        if name:
            self._s.command_expect(f'name {name}')
        if ip_addr:
            self._s.command_expect(f'ip-addr {ip_addr}')
        return self



from mdc.router.cli.rp_base.ng_router import config
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigIfLan(config.Config):
    """
    Class for configuring LAN interface.

        Args:
            session (Session): Session object to use for communication with the device.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (tuple): Tuple of positional arguments.
            _kwargs (dict): Dictionary of keyword arguments. 
    """

    def __init__(self, session: object, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
        Set the default prompts for the session.
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        """
        Get into the interface lan.
        """
        self._s.command_expect('interface lan')

    def getin(self):
        """
        Get into the interface lan and set the default prompts for the session.

            Returns: 
                object: The object instance. 
        """
        self._set_default_prompts()
        self._getin()
        return self

    def name(self, name: str):
        """
        Set a name for an interface or a VLAN group in a switch configuration file. 

            Args: 
                name (str): Name of an interface or a VLAN group to be set in switch configuration file.  

            Returns: 
                object: The object instance with updated parameters/attributes as per given name argument value.  
        """
        self._s.command_expect(f'name {name}')
        return self

    def set_lan_interface(self,
                            name: str = None,
                            ip: str = None,
                            mask: str = None,
                            bind_vlan: str = None,
                            mac_address: str = None):
        """
        Set the LAN interface of the device.

            Args:
                name (str): The name of the interface.
                ip (str): The IP address of the interface.
                mask (str): The subnet mask of the interface.
                bind_vlan (str): The VLAN to bind to the interface.
                mac_address (str): The MAC address of the interface.

            Returns:
                self: The object itself for method chaining. 
        """
        if name:
            self._s.command_expect(f'name {name}')
        if ip and mask:
            self._s.command_expect(f'ip address static {ip} {mask}')
        if bind_vlan:
            self._s.command_expect(f'bind vlan {bind_vlan}')
        if mac_address:
            self._s.command_expect(f'mac-address {mac_address}')
        return self

    def ip_address_static(self, ip: str, mask: str):
        """
        Set a static IP address on the device's LAN interface.

            Args: 
                ip (str): The IP address to set on the LAN interface. 
                mask (str): The subnet mask for the IP address being set on the LAN interface. 

            Returns: 
                self: The object itself for method chaining. 
        """
        self._s.command_expect(f'ip address static {ip} {mask}')
        return self

    def set_ospf(self, area_id: str = None, auth: dict = None):
        if area_id:
            self._s.command_expect(f'ip ospf area {area_id}')
        if auth:
            if auth['type'] == 'simple':
                self._s.command_expect(
                    f'ip ospf auth simple auth-key {auth["key"]}')
            elif auth['type'] == 'md5':
                self._s.command_expect(
                    f'ip ospf auth md5 {auth["key_id"]} auth-key {auth["key"]}'
                )
            else:
                raise ValueError(
                    f'input invalid authentication information: {auth}')
        return self



import sys

# from host import *
from behave import *
from host import HostFactory
from steps import login, interface, common


@when('login "{device}" via {username}')
def step_impl(context, device, username):
    context.dut_info[device]['credential']['username'] = username
    try:
        context.dut[device] = HostFactory.create_mdc_rp(
            context.dut_info[device], 
            debug_output=sys.stdout)
    except Exception as e:
        context.dut[device] = None


@then('"{device}" can be login successfully')
def step_impl(context, device):
    assert context.dut[device] != None, f'login failed!'


@then('check account privilege {action1} show system information on "{device}"')
def step_impl(context, action1, device):
    res = context.dut[device].main().show_system()
    if 'can' == action1:
        assert len(res) != 0, f'system information can not be shown!'
    elif 'can not' == action1:
        assert len(res) == 0, f'system information can be shown!'


@then('check account privilege {action2} enter configure level on "{device}"')
def step_impl(context, action2, device):
    try:
        res = context.dut[device].go_config()
    except Exception as e:
        res = None

    if 'can' == action2:
        assert res != None, f'configure level can not be entered!'
    elif 'can not' == action2:
        assert res == None, f'configure level can be entered!'


@then('check account privilege {action3} add a new user on "{device}"')
def step_impl(context, action3, device):
    try:
        res = context.dut[device].go_config().set_login_account(
            action="create",
            username="test",
            password="test",
            privilege="system admin")
    except Exception as e:
        res = None

    if 'can' == action3:
        assert res != None, f'new user can not be created!'
    elif 'can not' == action3:
        assert res == None, f'new user can be created!'



from utils.decorator import root_access_required
from session import SshSession
import time


class SnmpService:
    """
    A class to manage tftp server.

        Attributes:
            _ssh (SshSession): An instance of SshSession.
            _sudo_pwd (str): Password for sudo user.
    """

    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._sudo_pwd = sudo_paswd

    def _check_conneted(self):
        """
        Check if SSH session is connected.

        Raises:
            ValueError: If SSH session is  not connected.
        """
        if not self._ssh.is_connected():
            raise ValueError('SSH session not connected')

    @root_access_required
    def check_env(self):
        """
        Check the environment for the current SSH connection.

        Raises:
            ValueError: If the SNMP tool is not found.
        """
        self._check_conneted()

        ret = self._ssh.command('snmpwalk --version', timeout=5)
        if 'not found' in ret:
            ret = self._ssh.command('apt install -y snmp', timeout=120)
            assert 'fail' not in ret.lower(), f'snmp install failed.'

        ret = self._ssh.command('snmpget --version', timeout=5)
        if 'not found' in ret:
            self._ssh.command('apt install -y snmp', timeout=120)
            assert 'fail' not in ret.lower(), f'snmp install failed.'

    def get_snmp(self,
                 oid: str,
                 version: str,
                 username: str = 'admin',
                 priv_method: str = None,
                 community: str = 'public',
                 security: str = 'authPriv',
                 auth_key: str = 'auth_key',
                 priv_key: str = 'priv_key',
                 auth_type: str = 'no-auth',
                 agent_id: str = '192.168.127.254'):
        """
        Send SNMP get request to the specific SNMP agent.

            Attributes:
                oid (str): The Object Identifier (OID) to retrieve.
                version (str): SNMP version to use. Options: '1', '2c', '3'.
                username (str): SNMP username. Default is 'admin'.
                priv_method (str): SNMPv3 privacy protocol. Options: 'des', 'aes'. Default is None.
                community (str): SNMP community string. Options: 'public', 'private'. Default is 'public'.
                security (str): SNMPv3 security level. Options: 'authPriv', 'authNoPriv', 'NoAuthNoPriv'. Default is 'authPriv'.
                auth_key (str): SNMPv3 authentication key.  Default is 'auth_key'.
                priv_key (str): SNMPv3 privacy key. Default is 'priv_key'.
                auth_type (str): SNMPv3 authentication type. Options: 'md5', 'sha'. Default is 'no-auth'.
                agent_id (str): IP address of the SNMP agent. Default is '192.168.127.254'.
        
            Example:
                v1, v2c cmd: snmpwalk -v 1 -c public 192.168.127.254 1.3.6.1.2.1.1.5.0
                v3 cmd: snmpwalk -v 3 -l authPriv -u admin -a MD5 -A auth_key -x DES -X priv_key 192.168.127.254 1.3.6.1.2.1.1.5.0
        """
        cmd = 'snmpwalk '

        ## for SNMPv1, SNMPv2c
        if version == '1' or version == '2c':
            print(f'SNMP version {version}')
            cmd += f'-v {version} '
            cmd += f'-c {community} '

        ## for SNMPv3
        if version == '3':
            print('SNMP version 3')
            cmd += f'-v 3 '
            cmd += f'-l {security} '
            cmd += f'-u {username} '
            if auth_type != 'no-auth':
                cmd += f'-a {auth_type.upper()} -A {auth_key} '
                if priv_method:
                    cmd += f'-x {priv_method.upper()} -X {priv_key} '

        cmd += f'{agent_id} {oid}'
        time.sleep(5)

        ret = self._ssh.command(cmd)

        return ret

    def set_snmp(self,
                 oid: str,
                 version: str,
                 new_info: str,
                 username: str = 'admin',
                 priv_method: str = None,
                 community: str = 'public',
                 security: str = 'authPriv',
                 auth_key: str = 'auth_key',
                 priv_key: str = 'priv_key',
                 auth_type: str = 'no-auth',
                 agent_id: str = '192.168.127.254'):
        """
        Send SNMP set request to the specific SNMP agent.

            Attributes:
                oid (str): The Object Identifier (OID) to retrieve.
                version (str): SNMP version to use. Options: '1', '2c', '3'.
                new_info (str): New information related to changes in the OID.
                username (str): SNMP username. Default is 'admin'.
                priv_method (str): SNMPv3 privacy protocol. Options: 'des', 'aes'. Default is None.
                community (str): SNMP community string. Options: 'public', 'private'. Default is 'public'.
                security (str): SNMPv3 security level. Options: 'authPriv', 'authNoPriv', 'NoAuthNoPriv'. Default is 'authPriv'.
                auth_key (str): SNMPv3 authentication key.  Default is 'auth_key'.
                priv_key (str): SNMPv3 privacy key. Default is 'priv_key'.
                auth_type (str): SNMPv3 authentication type. Options: 'md5', 'sha'. Default is 'no-auth'.
                agent_id (str): IP address of the SNMP agent. Default is '192.168.127.254'.
                
            Example:
                v1, v2c cmd: snmpset -v 1 -c private 192.168.127.254 1.3.6.1.2.1.1.5.0 s "snmp_check"
                v3 cmd: snmpset -v 3 -l authPriv -u admin -a MD5 -A auth_key -x DES -X priv_key 192.168.127.254 1.3.6.1.2.1.1.5.0 s "snmp_check"
        """
        cmd = 'snmpset '

        ## for SNMPv1, SNMPv2c
        if version == '1' or version == '2c':
            print(f'SNMP version {version}')
            cmd += f'-v {version} '
            cmd += f'-c {community} '

        ## for SNMPv3
        if version == '3':
            print('SNMP version 3')
            cmd += f'-v 3 '
            cmd += f'-l {security} '
            cmd += f'-u {username} '
            if auth_type != 'no-auth':
                cmd += f'-a {auth_type.upper()} -A {auth_key} '
                if priv_method:
                    cmd += f'-x {priv_method.upper()} -X {priv_key} '

        cmd += f'{agent_id} {oid} s "{new_info}"'

        ret = self._ssh.command(cmd)

        time.sleep(10)

        return ret

    @root_access_required
    def start_snmptrapd(self):
        ret = self._ssh.command('dpkg -l | grep snmptrapd', timeout=5)

        if 'Net-SNMP notification receiver' not in ret.strip(''):
            ret = self._ssh.command('apt install -y snmptrapd', timeout=120)
            assert 'fail' not in ret.lower(), f'snmptrapd install failed.'

        ret = self._ssh.command('systemctl start snmptrapd', timeout=5)

        if 'Failed to start' in ret:
            raise ValueError(ret)



from host import *
from behave import *
from steps import login, interface, common

@when('"{host}" send UDP traffic to destination ip and port')
def step_impl(context, host):
    print(
        f'send traffic: {context.host_info[host]["testbed"]["cur_host"]} {context.table[0]["nat_dest_ip"]}:{context.table[0]["nat_dest_port"]}'
    )
    context.hosts[host].hping3.send( \
        tcpudp_flags=['udp'], \
        host = context.table[0]['nat_dest_ip'], \
        count = 4, \
        port = context.table[0]['nat_dest_port'])
    time.sleep(5)


@then('check "{host}" receive udp traffic with destination IP and port')
def step_impl(context, host):
    context.hosts[host].tshark.stop()

    ret = context.hosts[host].tshark.retrieve()

    ret_lines = ret.split('\n')

    count = 0
    # parse and get dest IP counts
    for a in ret_lines:
        # skip if not UDP
        if a.find("UDP") == -1:
            continue

        arr = a.lstrip(' ').split(' ')

        # skip unexpected format
        if arr[3] != '→' and arr[8] != '→':
            raise ValueError(f'unexpected tshark format {a}')

        if arr[4] == context.table[0]['real_dest_ip'] and arr[9] == context.table[0]['real_dest_port']:
            count += 1

    assert count >= 1 , \
        f'expect receive 4 number of udp but {count}, {ret}'


import time

from behave import *
from mdc.router.cli.rp_base.ng_router.base import Base
from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000

@when('{action} global firewall rule on "{device}"')
@given('{action} global firewall rule on "{device}"')
def step_impl(context, device, action):
    '''
    action: enable/disable
    '''
    context.dut[device].go_config().set_global_l3l7_policy(global_action=action).\
                                        set_global_l3l7_policy(default_action="allow")
    if action == "enable":
        time.sleep(10)


def set_firewall_rule(context, specific_protocols, specified_filter_mode,
                      in_itf, out_itf, host1, host2, device, model):
    if issubclass(model, tn5000):
        # get policy information
        src_ip = context.host_info[host1]['testbed']['cur_host']
        dst_ip = context.host_info[host2]['testbed']['cur_host']
        src_mac = context.host_info[host1]['testbed']['mac_address']

        if src_ip == "":
            src_ip = context.host_info[host1]['testbed'][
                f'{in_itf.lower()}_host']
        if dst_ip == "":
            dst_ip = context.host_info[host2]['testbed'][
                f'{out_itf.lower()}_host']
        print(f'{host1}: {src_ip}')
        print(f'{host2}: {dst_ip}')
        # set firewall log
        context.dut[device].go_config().set_log("firewall")
        # set filter mode
        mode = {
            "IP and Port Filtering": "ip",
            "Source MAC": "mac"
        }[specified_filter_mode]
        print("\nMode:", specified_filter_mode)

        # set firewall policy
        if mode == "ip":
            context.dut[device].go_config_firewall(1).set_firewall_policy(
                action="drop",
                mode=mode,
                protocol=specific_protocols.upper(),
                itf={
                    'in': in_itf.upper(),
                    'out': out_itf.upper()
                },
                sip=src_ip,
                dip=dst_ip,
                enable_log=["flash"])

        elif mode == "mac":
            print("Source Mac:", src_mac)
            context.dut[device].go_config_firewall(1).set_firewall_policy(
                action="drop",
                mode=mode,
                protocol=specific_protocols.upper(),
                itf={
                    'in': in_itf.upper(),
                    'out': out_itf.upper()
                },
                smac=src_mac,
                enable_log=["flash"])

    elif issubclass(model, Base):
        # set object...
        # set l37policy...
        src_ip = context.host_info[host1]['testbed']['cur_host']
        dst_ip = context.host_info[host2]['testbed']['cur_host']
        if src_ip == "":
            src_ip = context.host_info[host1]['testbed'][
                f'{in_itf.lower()}_host']
        if dst_ip == "":
            dst_ip = context.host_info[host2]['testbed'][
                f'{out_itf.lower()}_host']
        print(f'{host1}: {src_ip}')
        print(f'{host2}: {dst_ip}')

        context.dut[device].go_config_object_addr().create_object(
            name=f"src_ip_{src_ip.split('.')[2]}", ip_addr=src_ip)
        context.dut[device].go_config_object_addr().create_object(
            name=f"dst_ip_{dst_ip.split('.')[2]}", ip_addr=dst_ip)

        context.dut[device].go_config_object_serv(specific_protocols.lower()). \
                            set_object_serv(name=specific_protocols.upper())

        mode = {
            "IP and Port Filtering": "ip",
            "IP and Source MAC": "ip-mac",
            "Source MAC": "mac"
        }[specified_filter_mode]
        mode_rule = {
            "IP and Port Filtering": "IP/Port Filtering",
            "IP and Source MAC": "IP/Source Source MAC",
            "Source MAC": "Source MAC"
        }[specified_filter_mode]

        print("mode", mode)
        print("mode_rule", mode_rule)
        print("src_mac", context.dut_info[device]['mac_address'])

        if in_itf.upper() == 'BRG':
            in_itf = f'{in_itf}_LAN'
        if out_itf.upper() == 'BRG':
            out_itf = f'{out_itf}_LAN'
        if mode == "ip":
            context.dut[device].go_config().set_log(log_item="l3l7-policy")
            context.dut[device].go_config_l37policy().set_l37_policy(
                name=f"deny_{specific_protocols}_{mode}",
                enable=True,
                logging="enable",
                itf={
                    'in': in_itf.upper(),
                    'out': out_itf.upper()
                },
                policy_action="deny",
                mode=mode,
                sip=f"src_ip_{src_ip.split('.')[2]}",
                dip=f"dst_ip_{dst_ip.split('.')[2]}",
                dport=specific_protocols.upper())
            r = context.dut[device].main().show_l37_policy()
            rule_setting = [
                r[0]['incoming_interface'], r[0]['outgoing_interface'],
                r[0]['source_ip_address'], r[0]['destination_ip_address'],
                r[0]['destination_port_or_protocol'], r[0]['filter_mode'],
                r[0]['action'], r[0]['log']
            ]
            rule_expect = [
                in_itf.upper(),
                out_itf.upper(), f"src_ip_{src_ip.split('.')[2]}",
                f"dst_ip_{dst_ip.split('.')[2]}",
                specific_protocols.upper(), f"{mode_rule}", 'Deny', 'Enable'
            ]
            assert rule_setting == rule_expect, \
            f'polic setting dismatch: expect "{rule_expect}" but "{rule_setting}"'

        elif mode == "ip-mac":
            context.dut[device].go_config().set_log(log_item="l3l7-policy")
            context.dut[device].go_config_l37policy().set_l37_policy(
                name=f"deny_{specific_protocols}_{mode}",
                enable=True,
                logging="enable",
                itf={
                    'in': in_itf.upper(),
                    'out': out_itf.upper()
                },
                policy_action="deny",
                mode=mode,
                sip=f"src_ip_{src_ip.split('.')[2]}",
                smac=context.host_info[host1]['testbed']['mac_address'])
            r = context.dut[device].main().show_l37_policy()
            print(r)

        elif mode == "mac":
            context.dut[device].go_config().set_log(log_item="l3l7-policy")
            context.dut[device].go_config_l37policy().set_l37_policy(
                name=f"deny_{specific_protocols}_{mode}",
                enable=True,
                logging="enable",
                itf={
                    'in': in_itf.upper(),
                    'out': out_itf.upper()
                },
                policy_action="deny",
                mode=mode,
                smac=context.host_info[host1]['testbed']['mac_address'])
            r = context.dut[device].main().show_l37_policy()
            print(r)


@given(
    u'set a firewall rule on "{device}" to deny {specific_protocols} and "{specified_filter_mode}" from {in_itf} ("{host1}") to {out_itf} ("{host2}") and enable logging flash'
)
@when(
    u'set a firewall rule on "{device}" to deny {specific_protocols} and "{specified_filter_mode}" from {in_itf} ("{host1}") to {out_itf} ("{host2}") and enable logging flash'
)
def step_impl(context, specific_protocols, specified_filter_mode, in_itf,
              out_itf, host1, host2, device):
    set_firewall_rule(context, specific_protocols, specified_filter_mode,
                      in_itf, out_itf, host1, host2, device,
                      context.dut[device]._model)


@when(u'{status} the firewall rule with index {index} on "{device}"')
@given(u'{status} the firewall rule with index {index} on "{device}"')
def step_impl(context, device, index, status):
    '''
    status: enable/disable
    '''
    if issubclass(context.dut[device]._model, tn5000):
        context.dut[device].go_config().set_firewall_rule_status(
            context, index, status)
    elif issubclass(context.dut[device]._model, Base):
        context.dut[device].go_config().set_l37_rule_status(
            context, index, status)


def show_logging_event_log_firewall(context, device, model):
    if issubclass(model, tn5000):
        ret = context.dut[device].main().show_logging_event_log_firewall()
    else:
        ret = context.dut[device].main().show_logging_event_log_l3l7()
    return ret


@then(
    u'"{device}" can record firewall logs block {specific_protocols} traffic from {in_itf} ("{host1}") to {out_itf} ("{host2}")'
)
def step_impl(context, device, specific_protocols, in_itf, host1, out_itf,
              host2):
    ret = show_logging_event_log_firewall(context, device,
                                          context.dut[device]._model)
    count = 0
    if in_itf.upper() == 'BRG' and out_itf.upper() == 'BRG':
        in_itf, out_itf = 'BRG_LAN', 'BRG_LAN'

    if issubclass(context.dut[device]._model, tn5000):
        log_expect = [specific_protocols.upper(), in_itf.upper(), context.host_info[host1]['testbed']['cur_host'], out_itf.upper(), \
             context.host_info[host2]['testbed']['cur_host'], 'DROP']
    else:
        log_expect = [specific_protocols.upper(), in_itf.upper(), context.host_info[host1]['testbed']['cur_host'], out_itf.upper(), \
             context.host_info[host2]['testbed']['cur_host'], 'Deny']
    for r in ret:
        log_actual = [
            r['protocol'], r['in'], r['src_ip'], r['out'], r['dest_ip'],
            r['action']
        ]
        if log_expect == log_actual:
            count += 1
    if count == 0:
        assert False, '0 message lines logged'
    if specific_protocols.upper() == 'ICMP':
        assert count == 1, f'logged event-log dismatch: expect 1 but {count}, {log_expect} <-> {log_actual}'
    else:
        assert count >= 1, f'logged event-log dismatch: expect 4 but {count}, {log_expect} <-> {log_actual}'


@when(u'send traffic with {specific_protocols} from "{host1}" to "{host2}"')
def step_impl(context, specific_protocols, host1, host2):
    context.hosts[host2].tshark.start(interface=context.host_info[host2]['nic'])
    time.sleep(5)
    print(
        f'send traffic: {context.host_info[host1]["testbed"]["cur_host"]} {context.host_info[host2]["testbed"]["cur_host"]}'
    )
    context.hosts[host1].hping3.send( \
        tcpudp_flags=[specific_protocols.lower()], \
        host = context.host_info[host2]['testbed']['cur_host'], \
        count = 4)
    time.sleep(5)


@then(
    u'"{host1}" {can_or_cannot} receive traffic with {specific_protocols} from "{host2}"'
)
@then(
    u'check if "{host1}" {can_or_cannot} receive traffic with {specific_protocols} from "{host2}"'
)
def step_impl(context, host1, can_or_cannot, specific_protocols, host2):
    # ['ICMP' , 'Flags [S]', 'UDP']
    flag = { \
        'can': True, \
        'cannot': False}[can_or_cannot]
    context.hosts[host1].tshark.stop()
    ret = context.hosts[host1].tshark.retrieve()
    print(ret)
    count = 0
    if ret != None:
        print(
            f'{context.host_info[host2]["testbed"]["cur_host"]} → {context.host_info[host1]["testbed"]["cur_host"]} {specific_protocols.upper()}'
        )
        count = ret.count(
            f'{context.host_info[host2]["testbed"]["cur_host"]} → {context.host_info[host1]["testbed"]["cur_host"]} {specific_protocols.upper()}'
        )
    if flag:
        assert count >= 1 , \
        f'expect receive 4 number of {specific_protocols.upper()} but {count}, {ret}'
    else:
        assert count == 0, \
        f'expect receive 0 number of {specific_protocols.upper()} but {count}, {ret}'



from host import *
from behave import *
from steps.login import loginToDUT
from steps import common, interface, login
import time


@given(
    u'create "{tcp_connection}" with destination "{host}" and limitation "{conn_num}" on "{device}"'
)
def step_impl(context, tcp_connection, host, conn_num, device):
    '''
        Sessoion control rule will be like:
        index 1 session_1 dst_ip: dst_lan, ttl_tcp_conn: 10
    '''
    dst_ip = context.host_info[host]["testbed"]["cur_host"]
    obj_name = f'dst_{host}'
    # cerate object
    context.object = context.dut[device].go_config_object_addr().create_object(
        name=obj_name, ip_addr=dst_ip)

    context.dut[device].main().command('show object')

    # create session control policy
    context.session_name = f'session_{int(time.time())}'
    context.tcp_connection = tcp_connection
    context.index = 1
    if tcp_connection == 'total_tcp_connection':
        context.dut[device].go_config_session_ctrl().set_session_ctrl(
            name=context.session_name,
            enable=True,
            logging="enable",
            policy_action="deny",
            dip=obj_name,
            dport=None,
            ttl_tcp_conn=conn_num)
    elif tcp_connection == 'concurrent_tcp_conn':
        context.dut[device].go_config_session_ctrl().set_session_ctrl(
            name=context.session_name,
            enable=True,
            logging="enable",
            policy_action="deny",
            dip=obj_name,
            dport=None,
            concur_tcp_conn=conn_num)

    context.dut[device].main().command('show session-control')


@then(u'the session control rule is successfully {status} on "{device}"')
def step_impl(context, status, device):
    ret = context.dut[device].main().show_session_ctrl()
    flag = 0
    for i in ret:
        context.index = i['index']
        print(context.index)
        if i['session_name'] == context.session_name:
            flag = 1
            break
    if status == 'created':
        assert flag, f"the {context.session_name} session is not created"
    elif status == 'deleted':
        assert not flag, f"the {context.session_name} session is not deleted"


@when(u'create "{ssh_num}" TCP connection from "{host1}" to "{host2}"')
def step_impl(context, ssh_num, host1, host2):
    server_ip = context.host_info[host2]["testbed"]["cur_host"]
    server_account = context.host_info[host2]['session']["credential"][
        "username"]
    server_password = context.host_info[host2]['session']["credential"][
        "password"]

    context.hosts[host2].tshark.start(interface=context.host_info[host2]['nic'])
    time.sleep(2)

    context.hosts[host1].dut_ui.cli_init()
    for i in range(int(ssh_num)):
        print(f'this is {i} time to establish ssh session')
        context.hosts[host1].dut_ui.cli_cmd(
            f'''/bin/bash -c 'sshpass -p {server_password} ssh -o "StrictHostKeyChecking no" {server_account}@{server_ip}' ''',
            detach=True)
    time.sleep(2)
    context.hosts[host1].dut_ui.cli_destroy()


@when(
    u'send "{ssh_num}" TCP syn from "{host1}" to "{host2}" within 1 sec'
)
def step_impl(context, ssh_num, host1, host2):
    server_ip = context.host_info[host2]["testbed"]["cur_host"]
    server_account = context.host_info[host2]['session']["credential"][
        "username"]
    server_password = context.host_info[host2]['session']["credential"][
        "password"]

    context.hosts[host2].tshark.start(interface=context.host_info[host2]['nic'])
    time.sleep(2)

    print('== Trying to establish ssh session ==')
    context.hosts[host1].hping3.send( \
        tcpudp_flags=['syn'], \
        host = context.host_info[host2]['testbed']['cur_host'], \
        count = ssh_num, \
        port = 22)
    time.sleep(5)


@then(
    u'"{ssh_num}" TCP connection will be established from "{host1}" to "{host2}"'
)
def step_impl(context, ssh_num, host1, host2):
    context.hosts[host2].tshark.stop()
    ret = context.hosts[host2].tshark.retrieve()
    print(ret)
    count = 0
    if ret != None:
        print(
            f'{context.host_info[host2]["testbed"]["cur_host"]} → {context.host_info[host1]["testbed"]["cur_host"]} [SYN]'
        )  #
        count = ret.count(
            f'{context.host_info[host2]["testbed"]["cur_host"]} → {context.host_info[host1]["testbed"]["cur_host"]}' and \
            '→ 22 [SYN]')

    assert count == int(
        ssh_num
    ), f'expect receive {ssh_num} number of SYN but receive {count}, {ret}'



import time
from math import *
from datetime import *

from host import *
from behave import *
from steps.login import *
from steps.common import *
from steps.interface import *

TIME_PATTERN = "%a %b %d %H:%M:%S %Y"
TIME_NOW = None
CLOCK_SOURCE = None
REMOTE_SERVER = None


def set_clock(context, device):
    cfg = context.dut[device].go_config()
    cfg.TimeSync(cfg.get_session()).clock_source(CLOCK_SOURCE, REMOTE_SERVER)
    ret = context.dut[device].main().show_clock()
    assert ret['Clock Source'].lower() == CLOCK_SOURCE.lower(
    ), f'Setting clock source failed!'


def get_time_from_str(context, device):
    # Pattern looks like e.g., Mon Dec 19 11:10:13 2022
    ret = context.dut[device].main().show_clock()
    return datetime.strptime(ret['Current Time'], TIME_PATTERN)


def polling_time_sync(context, device, target_time):
    time_sync_timeout = 180  # in sec
    wait_interval = 5  # in sec
    tolerance = 60  # in sec
    # Algorithm: polling mode
    start = end = time.time()
    while (end - start) < time_sync_timeout:
        time_query = get_time_from_str(context, device)
        time_diff_sec = round(abs((time_query - target_time).total_seconds()))
        if time_diff_sec < tolerance:
            break
        else:
            time.sleep(wait_interval)
            end = time.time()
    else:
        print(
            f"target time: {target_time.strftime(TIME_PATTERN)}, get time: {time_query.strftime(TIME_PATTERN)}, tolerance: {tolerance}"
        )
        raise Exception(f"Time sync failed!")


@given('set ntp server enabled on "{device}"')
def step_impl(context, device):
    cfg = context.dut[device].go_config()
    cfg.TimeSync(cfg.get_session()).ntp_server_toggle(True)
    ret = context.dut[device].main().show_clock()
    assert ret['NTP/SNTP Server'].lower() == "Enabled".lower(
    ), f'Setting NTP/SNTP Server failed!'


@when('set clock source to "{clk_src}" on "{device}"')
def step_impl(context, clk_src, device):
    global CLOCK_SOURCE
    CLOCK_SOURCE = clk_src
    if CLOCK_SOURCE.lower() == 'local':
        set_clock(context, device)


@when('set remote server to "{server}"\'s LAN IP address on "{device}"')
def step_impl(context, server, device):
    global REMOTE_SERVER
    REMOTE_SERVER = context.dut_info[server]['testbed']['lan_ip']
    if CLOCK_SOURCE.lower() != 'local':
        set_clock(context, device)


@when('set current time to "{device}"')
@given('set current time to "{device}"')
def step_impl(context, device):
    global TIME_NOW
    TIME_NOW = datetime.utcnow()
    cfg = context.dut[device].go_config()
    cfg.TimeSync(cfg.get_session()).set_time_manually(TIME_NOW.year, TIME_NOW.month, TIME_NOW.day, \
        TIME_NOW.hour, TIME_NOW.minute, TIME_NOW.second)
    read_back = get_time_from_str(context, device)
    assert round(abs(
        (TIME_NOW -
         read_back).total_seconds())) <= 1, f"set current time failed!"


@then('"{device}" displays current time with accuracy in minute')
def step_impl(context, device):
    polling_time_sync(context, device, TIME_NOW)


@when('set "{num}" days prior to current time on "{device}"')
def step_impl(context, num, device):
    # Prepare time to set
    time_set = TIME_NOW - timedelta(days=int(num))
    # Set time
    cfg = context.dut[device].go_config()
    cfg.TimeSync(cfg.get_session()).set_time_manually(time_set.year, time_set.month, time_set.day, \
        time_set.hour, time_set.minute, time_set.second)


@then('"{device}" displays correct date with "{num}" days prior to current date'
     )
def step_impl(context, device, num):
    target = TIME_NOW - timedelta(days=int(num))
    polling_time_sync(context, device, target)
